function analyzeText()
  function count_all_elements(text)
    local count = 0
    for _ in utf8.codes(text) do
      count = count + 1
    end
    return count
  end


  function is_numeric_arabic(codepoint)
    local numeric_arabic = {
      [0x0660] = true, -- ٠
      [0x0661] = true, -- ١
      [0x0662] = true, -- ٢
      [0x0663] = true, -- ٣
      [0x0664] = true, -- ٤
      [0x0665] = true, -- ٥
      [0x0666] = true, -- ٦
      [0x0667] = true, -- ٧
      [0x0668] = true, -- ٨
      [0x0669] = true, -- ٩
    }
    return numeric_arabic[codepoint]
  end

  function count_characters(text)
    local count = 0
    for _, codepoint in utf8.codes(text) do
      if ((codepoint >= 0x0041 and codepoint <= 0x005A) or -- A-Z
        (codepoint >= 0x0061 and codepoint <= 0x007A) or -- a-z
        (codepoint >= 0x00C0 and codepoint <= 0x00FF) or -- Latin-1 Supplement
        (codepoint >= 0x0600 and codepoint <= 0x06FF and not is_numeric_arabic(codepoint))) then -- Arabic
        if not (codepoint == 0x060C or -- ،
          codepoint == 0x061F or -- ؟
          codepoint == 0x061B) then -- ؛
          count = count + 1
        end
      end
    end
    return count
  end

  function count_arabic_characters(text)
    local count = 0
    for _, codepoint in utf8.codes(text) do
      if (codepoint >= 0x0600 and codepoint <= 0x06FF and not is_numeric_arabic(codepoint)) then -- Arabic
        if not (codepoint == 0x060C or -- ،
          codepoint == 0x061F or -- ؟
          codepoint == 0x061B) then -- ؛
          count = count + 1
        end
      end
    end
    return count
  end


  function count_latin_characters(text)
    local count = 0

    for _, codepoint in utf8.codes(text) do
      if ((codepoint >= 0x0041 and codepoint <= 0x005A) or -- A-Z
        (codepoint >= 0x0061 and codepoint <= 0x007A) or -- a-z
        (codepoint == 0x00C9) or -- É
        (codepoint == 0x00E8)) then -- è

        count = count + 1
      end
    end

    return count
  end


  function count_uppercase_latin_characters(text)
    local count = 0

    for _, codepoint in utf8.codes(text) do
      if (codepoint >= 0x0041 and codepoint <= 0x005A) then -- A-Z
        count = count + 1
      end
    end

    return count
  end

  local text = service.getText(node)


  function count_lowercase_latin_characters(text)
    local count = 0

    for _, codepoint in utf8.codes(text) do
      if (codepoint >= 0x0061 and codepoint <= 0x007A) then -- a-z
        count = count + 1
      end
    end

    return count
  end

  function count_words(text)
    -- Replace non-breaking spaces (UTF-8: \194\160) with regular spaces
    local text_with_spaces = text:gsub("\194\160", " ")

    -- Count words using pattern that matches sequences of non-space characters
    local _, word_count = text_with_spaces:gsub("%S+", "")

    return word_count
  end


  function count_lines(text)
    local _, line_count = text:gsub("\n", "\n")
    return line_count + 1
  end

  function count_paragraphs(text)
    local _, paragraph_count = text:gsub("\n\n+", "\n\n")
    return paragraph_count + 1
  end

  function count_sentences(text)
    local _, sentence_count = text:gsub("[%.%!%?]+", "")
    return sentence_count
  end

  function count_numbers(text)
    local _, number_count = text:gsub("%d", "")
    return number_count
  end

  function count_arabic_numbers(text)
    local count = 0
    for _, codepoint in utf8.codes(text) do
      if codepoint >= 0x0660 and codepoint <= 0x0669 then -- Arabic digits 0-9
        count = count + 1
      end
    end
    return count
  end



  local allowed_punctuations = {
    ["."] = true, [","] = true, [";"] = true, [":"] = true,
    ["!"] = true, ["?"] = true, ["؟"] = true, ["،"] = true,
    ["؛"] = true, ["("] = true, [")"] = true, ["["] = true,
    ["]"] = true, ["{"] = true, ["}"] = true, ['"'] = true,
    ["'"] = true, ["-"] = true, ["–"] = true, ["—"] = true,
    ["_"] = true, ["…"] = true, ["«"] = true, ["»"] = true,
    ["‹"] = true, ["›"] = true, ["~"] = true, ["`"] = true,
    ["@"] = true, ["#"] = true, ["$"] = true, ["%"] = true,
    ["^"] = true, ["&"] = true, ["*"] = true, ["="] = true,
    ["+"] = true, ["|"] = true, ["\\"] = true, ["/"] = true,
    ["<"] = true, [">"] = true
  }


  function count_punctuation(text)
    local punctuation_count = 0
    local utf8_char_pattern = "[%z\1-\127\194-\244][\128-\191]*"

    for uchar in text:gmatch(utf8_char_pattern) do
      if allowed_punctuations[uchar] then
        punctuation_count = punctuation_count + 1
      end
    end

    return punctuation_count
  end

  function get_unique_punctuation(text)
    local unique_punctuations = {}
    local seen_punctuations = {}
    local utf8_char_pattern = "[%z\1-\127\194-\244][\128-\191]*"

    for uchar in text:gmatch(utf8_char_pattern) do
      if allowed_punctuations[uchar] and not seen_punctuations[uchar] then
        table.insert(unique_punctuations, uchar)
        seen_punctuations[uchar] = true
      end
    end

    return unique_punctuations
  end




  function count_emojis(text)
    local emoji_ranges = {
      {0x1F600, 0x1F64F}, -- Emoticons
      {0x1F300, 0x1F5FF}, -- Miscellaneous Symbols and Pictographs
      {0x1F680, 0x1F6FF}, -- Transport and Map Symbols
      {0x1F700, 0x1F77F}, -- Alchemical Symbols
      {0x1F780, 0x1F7FF}, -- Geometric Shapes Extended
      {0x1F800, 0x1F8FF}, -- Supplemental Arrows-C
      {0x1F900, 0x1F9FF}, -- Supplemental Symbols and Pictographs
      {0x1FA00, 0x1FA6F}, -- Chess Symbols
      {0x1FA70, 0x1FAFF}, -- Symbols and Pictographs Extended-A
      {0x2600, 0x26FF}, -- Miscellaneous Symbols
      {0x2700, 0x27BF}, -- Dingbats
      {0xFE00, 0xFE0F}, -- Variation Selectors
      {0x1F1E6, 0x1F1FF}, -- Regional Indicator Symbols
      {0x200D, 0x200D}, -- Zero Width Joiner
      {0x20E3, 0x20E3}, -- Combining Enclosing Keycap
      {0x1F018, 0x1F270}, -- Various Symbols
    }

    local emoji_count = 0

    for _, codepoint in utf8.codes(text) do
      for _, range in ipairs(emoji_ranges) do
        if codepoint >= range[1] and codepoint <= range[2] then
          emoji_count = emoji_count + 1
          break
        end
      end
    end

    return emoji_count
  end



  function count_diacritics(text)
    local diacritic_count = 0
    local diacritic_pattern = "[\216-\219][\128-\191]" -- Arabic diacritic UTF-8 range

    -- Adding manually provided Arabic diacritics
    local diacritics = "ًٌٍَُِّْ"
    for _, diacritic in utf8.codes(diacritics) do
      local diacritic_utf8 = utf8.char(diacritic)
      local _, count = text:gsub(diacritic_utf8, "")
      diacritic_count = diacritic_count + count
    end
    return diacritic_count
  end

  function detect_text_direction(text)
    local is_rtl = false -- افتراضيًا النص من اليسار إلى اليمين

    -- النمط الذي يعمل في اللغة العربية واللغات الأخرى التي تكتب من اليمين إلى اليسار
    local rtl_pattern = "[%z\1-\127\194-\244][\128-\191]*[\216-\219][\128-\191]*"

    if text:find(rtl_pattern) then
      is_rtl = true
    end

    return is_rtl
  end


  function detect_encoding(text)
    if text:sub(1, 3) == '\239\187\191' then
      return "utf-8"
     elseif text:sub(1, 2) == '\254\255' then
      return "utf-16-be"
     elseif text:sub(1, 2) == '\255\254' then
      return "utf-16-le"
     elseif text:sub(1, 4) == '\0\0\254\255' then
      return "utf-32-be"
     elseif text:sub(1, 4) == '\255\254\0\0' then
      return "utf-32-le"
    end

    local function is_utf8(s)
      return s:match("^[\194-\244][\128-\191]*$") or s:match("^[%z\1-\127]+$")
    end

    if is_utf8(text) then
      return "utf-8"
    end

    local function is_ascii(s)
      return s:match("^[%z\1-\127]*$")
    end

    if is_ascii(text) then
      return "ascii"
    end

    local function is_iso8859_1(s)
      return s:match("^[\0-\255]*$")
    end

    if is_iso8859_1(text) then
      return "iso-8859-1"
    end

    return "unknown"
  end




  require "import"
  text = service.getEditText().text
  service.setText("")
  task(300, function()
    text2 = service.getEditText().text
    if text == text2
      local handler = Handler(Looper.getMainLooper())
      this.postSpeak(200,"لا يوجد نص")
      local function checkSpeakingStatus()
        if not service.isSpeaking() then
          dictAr()
         else
          handler.postDelayed(checkSpeakingStatus, 1000) -- إعادة التحقق بعد ثانية
        end
      end
      handler.postDelayed(checkSpeakingStatus, 1500) -- بدء التحقق بعد 1.5 ثانية
     else
      service.setText(text)
      service.speak("يوجد نص")
      service.click({{"<عدد جميع العناصر*>5$2000","%Navigation automatique", }})
      task(1300, function()


        local text = text
        local all_elements_count = count_all_elements(text)
        local word_count = count_words(text)
        local line_count = count_lines(text)
        local character_count = count_characters(text)
        local arabic_character_count = count_arabic_characters(text)
        local latin_character_count = count_latin_characters(text)
        local uppercase_latin_character_count = count_uppercase_latin_characters(text)
        local lowercase_latin_character_count = count_lowercase_latin_characters(text)
        local paragraph_count = count_paragraphs(text)
        local sentence_count = count_sentences(text)
        local number_count = count_numbers(text)
        local arabic_number_count = count_arabic_numbers(text)
        local punctuation_count = count_punctuation(text)
        local unique_punctuation_list = get_unique_punctuation(text)
        local emoji_count = count_emojis(text)
        local diacritic_count = count_diacritics(text)
        local direction = detect_text_direction(text)
        local text_direction = "من اليمين إلى اليسار"
        if not direction then
          text_direction = "من اليسار إلى اليمين"
        end

        local encoding = detect_encoding(text)


        print("عدد جميع العناصر: " .. all_elements_count)
        print("جميعُ الأحرف: " .. character_count)
        print("الأحْرُفُ العربية: " .. arabic_character_count)
        print("الأحْرُفُ اللاتينية: " .. latin_character_count)
        print(" الكبيرة: " .. uppercase_latin_character_count)
        print(" الصغيرة: " .. lowercase_latin_character_count)
        print("الكلمات: " .. word_count)
        print("الْجُمَلْ: " .. sentence_count)
        print("الأسطر: " .. line_count)
        print("الفقرات: " .. paragraph_count)
        print("الأرقامُ العربية: " .. number_count)
        print("الأرقامُ الهندية:  " .. arabic_number_count)
        print("العلامات: " .. punctuation_count)
        print("أنواعُها:  " .. table.concat(unique_punctuation_list))
        print("الإيموجي: " .. emoji_count)
        print("التشكيل: " .. diacritic_count)
        print("اتجاه النص: " .. text_direction)
        print("ترميز النص: " .. encoding)

      end)
    end
  end)


end

analyzeText()