require "import"
import "android.hardware.SensorManager"
import "android.hardware.Sensor"
import "android.content.Context"
import "android.widget.Button"
import "android.widget.TextView"
import "android.widget.LinearLayout"
import "android.app.Dialog"
import "android.os.Vibrator"
import "android.content.Context"
import "android.location.LocationManager"
import "android.location.Geocoder"
import "java.util.Locale"
import "java.lang.Math"
import "java.io.File"

local filePath = "/storage/emulated/0/Documents/comfortableDictation/بيانات الاتجاهات.txt"

function saveLocationData(locationInfo)
    local file = io.open(filePath, "w")
    if file then
        file:write(string.format("%s\n%s\n%s\n%d", locationInfo.country, locationInfo.adminArea, locationInfo.subAdminArea, locationInfo.qiblaAngle))
        file:close()
    end
end

function loadLocationData()
    local file = io.open(filePath, "r")
    if file then
        local country = file:read("*l")
        local adminArea = file:read("*l")
        local subAdminArea = file:read("*l")
        local qiblaAngle = tonumber(file:read("*l")) or 105
        file:close()
service.postSpeak(2000,"تم جلب المعلومات من الملف" )
        return { country = country, adminArea = adminArea, subAdminArea = subAdminArea, qiblaAngle = qiblaAngle }
    end
    return nil
end


function calculateQiblaAngle(lat, lon)
    local kaabaLat, kaabaLon = 21.4225, 39.8262  -- إحداثيات الكعبة المشرفة
    local deltaLon = math.rad(kaabaLon - lon)
    
    local lat1 = math.rad(lat)
    local lat2 = math.rad(kaabaLat)
    
    local y = math.sin(deltaLon) * math.cos(lat2)
    local x = math.cos(lat1) * math.sin(lat2) - math.sin(lat1) * math.cos(lat2) * math.cos(deltaLon)
    local qiblaAngle = math.deg(math.atan2(y, x))
    
    return (qiblaAngle + 360) % 360
end


function getLocationInfo()
    local lm = service.getSystemService(Context.LOCATION_SERVICE)
    local provider = LocationManager.NETWORK_PROVIDER
    local location = lm.getLastKnownLocation(provider)
    
    if location then
        local geocoder = Geocoder(service, Locale.getDefault())
        local addresses = geocoder.getFromLocation(location.getLatitude(), location.getLongitude(), 1)
        
        if addresses and not addresses.isEmpty() then
            local address = addresses.get(0)
            local country = address.getCountryName() or "غير متوفر"
            local adminArea = address.getAdminArea() or "غير متوفر"
            local subAdminArea = address.getSubAdminArea() or address.getLocality() or "غير متوفر"
            local qiblaAngle = math.floor(calculateQiblaAngle(location.getLatitude(), location.getLongitude()) + 0.5)
            
            local locationInfo = { country = country, adminArea = adminArea, subAdminArea = subAdminArea, qiblaAngle = qiblaAngle }
            saveLocationData(locationInfo)
service.postSpeak(2000,"تم جلب المعلومات من الموقع" ..locationInfo.adminArea ..". " .. locationInfo.subAdminArea)
            return locationInfo
        end
    end
service.postSpeak(2000,"تعذر الحصول على الموقع، تأكد من تفعيل GPS وإعطاء الإذن.")
    return nil
end

local locationInfo = getLocationInfo() or loadLocationData() or { country = "غير متوفر", adminArea = "غير متوفر", subAdminArea = "غير متوفر", qiblaAngle = 105 }




local context = service or activity
sensorManager = context.getSystemService(Context.SENSOR_SERVICE)
gravity, geomagnetic, azimuth = {0, 0, 0}, {0, 0, 0}, 0
local isSpeaking = false
local isQiblaDetectionEnabled = false
local currentDirection = ""
local currentAzimuth = ""
local speakMode = "direction"
local speakTiming = "direction_change"
local lastAzimuth = 0
local lastDirection = ""
local isInQiblaRange = false
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
local lastVoiceGuidanceTime = 0
local locationInfo = getLocationInfo() or loadLocationData() or { country = "غير متوفر", adminArea = "غير متوفر", subAdminArea = "غير متوفر", qiblaAngle = "غير متوفر", }
local qiblaAngle = locationInfo.qiblaAngle

sensorEventListener = {  
    onSensorChanged = function(event)    
        if event.sensor.getType() == Sensor.TYPE_ACCELEROMETER then      
            gravity = event.values    
        elseif event.sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD then      
            geomagnetic = event.values    
        end    
        local R, I = float[9], float[9]    
        if SensorManager.getRotationMatrix(R, I, gravity, geomagnetic) then      
            local orientation = float[3]      
            SensorManager.getOrientation(R, orientation)      
            azimuth = (math.deg(orientation[0]) + 360) % 360      
            local newDirection = getDirection(azimuth)
            currentDirection = newDirection      
            currentAzimuth = tostring(math.floor(azimuth)) .. "°"      
            updateDirectionDisplay()
            
            if isSpeaking then
                if speakTiming == "angle_change" and math.abs(azimuth - lastAzimuth) >= 5 then
                    lastAzimuth = azimuth
                    speakDirection()
                elseif speakTiming == "direction_change" and newDirection ~= lastDirection then
                    lastDirection = newDirection
                    speakDirection()
                end
            end    
            
            if isQiblaDetectionEnabled then
                if math.abs(azimuth - qiblaAngle) <= 5 then
                    if not isInQiblaRange then
                        isInQiblaRange = true
                        vibrator.vibrate(100000)
                        qiblaButton.setBackgroundColor(0xFF00FF00)
                        service.speak("القِبْلَة")
                    end
                else
                    if isInQiblaRange then
                        isInQiblaRange = false
                        vibrator.cancel()
                        qiblaButton.setBackgroundColor(0xFFFF0000)
                    end
                    if os.time() - lastVoiceGuidanceTime >= 3 then
                        local difference = (azimuth - qiblaAngle + 360) % 360
                        if difference > 180 then
                            difference = difference - 360
                        end
                        if difference > 0 then
                            service.speak("يَسار")
                        else
                            service.speak("يمين")
                        end
                        lastVoiceGuidanceTime = os.time()
                    end
                end
            end
        end  
    end
}

sensorManager.registerListener(sensorEventListener, sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER), SensorManager.SENSOR_DELAY_UI)
sensorManager.registerListener(sensorEventListener, sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD), SensorManager.SENSOR_DELAY_UI)

function getDirection(angle)  
    local directions = {
        [0] = "شمال", [1] = "شمال شرق", [2] = "شرق", [3] = "جنوب شرق",
        [4] = "جنوب", [5] = "جنوب غرب", [6] = "غرب", [7] = "شمال غرب"
    }  
    return directions[math.floor((angle + 22.5) / 45) % 8]
end

function speakDirection()  
    if isSpeaking then    
        if speakMode == "angle" then
            service.speak(currentAzimuth)
        else
            service.speak(currentDirection)
        end    
        if speakTiming == "interval" then
            task(3000, speakDirection)
        end
    end
end

local directionTextView, azimuthTextView, countryTextView, adminAreaTextView, subAdminAreaTextView, qiblaAngleTextView

function updateDirectionDisplay()  
    if directionTextView and azimuthTextView then    
        directionTextView.setText("الاتجاه: " .. currentDirection)    
        azimuthTextView.setText("الزاوية: " .. currentAzimuth)  

        if currentDirection == "شرق" then
            directionTextView.setBackgroundColor(0xFF0000CD)
            azimuthTextView.setBackgroundColor(0xFF0000CD)
        elseif currentDirection == "غرب" then
            directionTextView.setBackgroundColor(0xFF8B0000)
            azimuthTextView.setBackgroundColor(0xFF8B0000)
        elseif currentDirection == "جنوب" then
            directionTextView.setBackgroundColor(0xFFCDCD00)
            azimuthTextView.setBackgroundColor(0xFFCDCD00)
        elseif currentDirection == "شمال" then
            directionTextView.setBackgroundColor(0xFF000000)
            azimuthTextView.setBackgroundColor(0xFF000000)
        elseif currentDirection == "شمال شرق" then
            directionTextView.setBackgroundColor(0xFF000033)
            azimuthTextView.setBackgroundColor(0xFF000033)
        elseif currentDirection == "جنوب شرق" then
            directionTextView.setBackgroundColor(0xFF87CEEB)
            azimuthTextView.setBackgroundColor(0xFF87CEEB)
        elseif currentDirection == "جنوب غرب" then
            directionTextView.setBackgroundColor(0xFF8B8B00)
            azimuthTextView.setBackgroundColor(0xFF8B8B00)
        elseif currentDirection == "شمال غرب" then
            directionTextView.setBackgroundColor(0xFF450000)
            azimuthTextView.setBackgroundColor(0xFF450000)
        else
            directionTextView.setBackgroundColor(0x00000000)
            azimuthTextView.setBackgroundColor(0x00000000)
        end
    end

    if countryTextView and adminAreaTextView and subAdminAreaTextView and qiblaAngleTextView then
        countryTextView.setText("الدولة: " .. locationInfo.country)
        adminAreaTextView.setText("المحافظة: " .. locationInfo.adminArea)
        subAdminAreaTextView.setText("المنطقة: " .. locationInfo.subAdminArea)
        qiblaAngleTextView.setText("زاوية القبلة: " .. locationInfo.qiblaAngle .. "°")
    end
end

function createControlDialog()  
    local dialog = LuaDialog(service)  
    dialog.setCancelable(false)

    dialog.setOnKeyListener(luajava.createProxy("android.content.DialogInterface$OnKeyListener", {
        onKey = function(dialog, keyCode, event)
            if keyCode == 4 then
                service.speak("إضْغَطْ على إغلاق")
                return true
            end
            return false
        end
    }))

    local layout = LinearLayout(context)  
    layout.setOrientation(LinearLayout.VERTICAL)    
    directionTextView = TextView(context)  
    directionTextView.setText("الاتجاه: " .. currentDirection)  
    directionTextView.setTextSize(40)
    layout.addView(directionTextView)    
    azimuthTextView = TextView(context)  
    azimuthTextView.setText("الزاوية: " .. currentAzimuth)  
    azimuthTextView.setTextSize(40)
    layout.addView(azimuthTextView)    

    countryTextView = TextView(context)
    countryTextView.setText("الدولة: " .. locationInfo.country)
    countryTextView.setTextSize(30)
countryTextView.setBackgroundColor(0xFF000000)
    layout.addView(countryTextView)

    adminAreaTextView = TextView(context)
    adminAreaTextView.setText("المحافظة: " .. locationInfo.adminArea)
    adminAreaTextView.setTextSize(30)
adminAreaTextView.setBackgroundColor(0xFF000000)
    layout.addView(adminAreaTextView)

    subAdminAreaTextView = TextView(context)
    subAdminAreaTextView.setText("المنطقة: " .. locationInfo.subAdminArea)
    subAdminAreaTextView.setTextSize(30)
subAdminAreaTextView.setBackgroundColor(0xFF000000)
    layout.addView(subAdminAreaTextView)

    qiblaAngleTextView = TextView(context)
    qiblaAngleTextView.setText("زاوية القبلة: " .. locationInfo.qiblaAngle .. "°")
    qiblaAngleTextView.setTextSize(30)
qiblaAngleTextView.setBackgroundColor(0xFF000000)
    layout.addView(qiblaAngleTextView)

    qiblaButton = Button(context)  
    qiblaButton.setText(isQiblaDetectionEnabled and "إيقافُ التَّعَرُّفِ على القِبْلَة" or "التَّعَرُّفُ على القِبْلَة")    
    qiblaButton.setBackgroundColor(isQiblaDetectionEnabled and 0xFFFF0000 or 0xFF000000)
    qiblaButton.setTextSize(30)
    qiblaButton.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {    
        onClick = function(view)      
        if qiblaAngle == "غير متوفر" or not qiblaAngle then
            service.speak("زاويةُ القِبْلَةِ غيرُ متوفرة، تأكَّد من تشغيل الموقع والبياناتْ.")
            return
        end

            isQiblaDetectionEnabled = not isQiblaDetectionEnabled      
            if isQiblaDetectionEnabled then        
                qiblaButton.setText("إيقافُ التَّعَرُّفِ على القِبْلَة")      
                qiblaButton.setBackgroundColor(0xFFFF0000)
            else        
                qiblaButton.setText("التَّعَرُّفُ على القِبْلَة")      
service.speak("تم ايقاف التعرف على القِبْلة")
                qiblaButton.setBackgroundColor(0xFF000000)
                vibrator.cancel()
            end    
        end
    }))


local updateButton = Button(context)
updateButton.setText("تحديث")
updateButton.setBackgroundColor(0xFF000000)
updateButton.setTextSize(30)

updateButton.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {
    onClick = function(view)
            isSpeaking = false  
            isQiblaDetectionEnabled = false
            vibrator.cancel()
            dialog.dismiss()    
getLocationInfo()
task(200, createControlDialog)
    end
}))




    local toggleButton = Button(context)  
    toggleButton.setText("نُطْقُ الإتِّجاهات")
    toggleButton.setBackgroundColor(0xFF000000)
    toggleButton.setTextSize(30)
    toggleButton.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {    
        onClick = function(view)      
            isSpeaking = not isSpeaking      
            if isSpeaking then        
                toggleButton.setText("إيقافُ نُطْقِ الإتِّجاهات")        
                speakDirection()      
            else        
                toggleButton.setText("نُطْقُ الإتِّجاهات")      
service.speak("تم ايقاف نطق الاتجاهات")
            end    
        end
    }))    
    local modeButton = Button(context)  
    modeButton.setText("حَسَبَ الزاوية") 
    modeButton.setBackgroundColor(0xFF000000)
    modeButton.setTextSize(30)
    modeButton.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {
        onClick = function(view)
            if speakMode == "angle" then
                speakMode = "direction"  
                modeButton.setText("حَسَبَ الزاوية") 
            else
                speakMode = "angle"  
                modeButton.setText("حَسَبَ الاتجاه") 
            end
        end
    }))
    local timingButton = Button(context)
    timingButton.setText("كُلَّ ثلاثِ ثوانٍ")
    timingButton.setBackgroundColor(0xFF000000)
    timingButton.setTextSize(30)
    timingButton.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {
        onClick = function(view)
            if speakTiming == "interval" then
                speakTiming = "angle_change"
                timingButton.setText("عِنْدَ تَغَيُّرِ الْإِتِّجَاه")
            elseif speakTiming == "angle_change" then
                speakTiming = "direction_change"
                timingButton.setText("كُلَّ ثلاثِ ثوانٍ")
            else
                speakTiming = "interval"
                timingButton.setText("عِنْدَ تَغَيُّرِ الزاوية")
                speakDirection()
            end
        end
    }))
    local closeButton = Button(context)  
    closeButton.setText("إغلاق")  
    closeButton.setBackgroundColor(0xFF000000)
    closeButton.setTextSize(30)
    closeButton.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {    
        onClick = function(view)      
            isSpeaking = false  
            isQiblaDetectionEnabled = false
            vibrator.cancel()
            dialog.dismiss()    
        end
    }))    
    layout.addView(qiblaButton)  
layout.addView(updateButton)
    layout.addView(toggleButton)  
    layout.addView(modeButton)  
    layout.addView(timingButton)  
    layout.addView(closeButton)  
    dialog.setView(layout)    

    dialog.show()
end

createControlDialog()