require "import"
import "android.content.Context"
import "android.location.LocationManager"
import "android.location.Geocoder"
import "java.util.Locale"
import "java.io.File"

local filePath = "/storage/emulated/0/Documents/comfortableDictation/بيانات الموقع.txt"

function saveLocationData(info)
  local file = io.open(filePath, "w")
  if file then
    file:write(string.format("%s\n%s\n%s\n%s\n%s", 
      info.country, info.adminArea, info.subAdminArea, 
      info.neighborhood or "غير متوفر", info.street or "غير متوفر"))
    file:close()
  end
end

function loadLocationData()
  local file = io.open(filePath, "r")
  if file then
    local country = file:read("*l")
    local adminArea = file:read("*l")
    local subAdminArea = file:read("*l")
    local neighborhood = file:read("*l")
    local street = file:read("*l")
    file:close()

    if country and adminArea and subAdminArea and neighborhood and street then
      service.postSpeak(2000, "تم جلب المعلومات من الملف: " ..
        street .. ". " .. neighborhood .. ". " .. subAdminArea .. ". " .. adminArea .. ". ")
      return {
        country = country,
        adminArea = adminArea,
        subAdminArea = subAdminArea,
        neighborhood = neighborhood,
        street = street
      }
    else
      service.postSpeak(2000, "لم يتم العثور على جميع المعلومات في الملف.")
    end
  else
    service.postSpeak(2000, "لا يوجد ملف يحتوي على بيانات. يرجى تشغيل الموقع والإنترنت.")
  end
  return nil
end

function getLocationInfo()
  local lm = service.getSystemService(Context.LOCATION_SERVICE)
  local provider = LocationManager.NETWORK_PROVIDER
  local location = lm.getLastKnownLocation(provider)
  if location then
    local geocoder = Geocoder(service, Locale.getDefault())
    local addresses = geocoder.getFromLocation(location.getLatitude(), location.getLongitude(), 1)
    if addresses and not addresses.isEmpty() then
      local addr = addresses.get(0)
      local country = addr.getCountryName() or "غير متوفر"
      local admin = addr.getAdminArea() or "غير متوفر"
      local subAdmin = addr.getSubAdminArea() or addr.getLocality() or "غير متوفر"
      local neighborhood = addr.getSubLocality() or "غير متوفر"

      local street = addr.getThoroughfare()
      if not street or street == "" then
        local feature = addr.getFeatureName()
        if feature and not feature:match("^%d+$") then
          street = feature
        else
          street = "غير متوفر"
        end
      end

      local data = {
        country = country,
        adminArea = admin,
        subAdminArea = subAdmin,
        neighborhood = neighborhood,
        street = street
      }
      saveLocationData(data)
      service.postSpeak(2000, "تم جلب المعلومات من الموقع: " ..
        street .. ". " .. neighborhood .. ". " .. subAdmin .. ". " .. admin .. ". ")
      return data
    end
  end
  service.postSpeak(2000, "تعذر الحصول على الموقع، سيتم استخدام البيانات المحفوظة.")
  return nil
end

local locationInfo = getLocationInfo() or loadLocationData() or {
  country = "غير متوفر",
  adminArea = "غير متوفر",
  subAdminArea = "غير متوفر",
  neighborhood = "غير متوفر",
  street = "غير متوفر"
}