function checkContactsAccess2()
  local activity = this
  local resolver = activity.getContentResolver()
  local hasPermission = activity.checkSelfPermission("android.permission.READ_CONTACTS") == activity.PackageManager.PERMISSION_GRANTED
  if not hasPermission then
    activity.requestPermissions({"android.permission.READ_CONTACTS"}, 1)
    return "Permission required to access contacts."
  end

  local cursor = resolver.query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
  {ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME, ContactsContract.CommonDataKinds.Phone.NUMBER},
  nil, nil, nil)

  local contactsList = {}

  if cursor then
    while cursor.moveToNext() do
      local name = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME))
      local number = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
      number = normalizeNumber(number)

      if contactsList[name] == nil then
        contactsList[name] = {}
      end

      if not table.contains(contactsList[name], number) then
        table.insert(contactsList[name], number)
      end
    end
    cursor.close()
   else
    return "Failed to retrieve contacts."
  end

  local uniqueContactsList = {}
  for name, numbers in pairs(contactsList) do
    table.insert(uniqueContactsList, {name = name, numbers = numbers})
  end

  return uniqueContactsList
end



-- Utility function to check if a value exists in a table
function table.contains(table, value)
  for _, v in ipairs(table) do
    if v == value then
      return true
    end
  end
  return false
end

function sortContacts(contactsList)
  table.sort(contactsList, function(a, b)
    return a.name < b.name -- ترتيب هجائي
  end)
end


function normalizeNumber(number)
  return number:gsub("%s+", "") -- إزالة المسافات
end


function displayContacts()
  local contactsList = checkContactsAccess2()
  local activity = this
  local layout = LinearLayout(activity)
  layout.setOrientation(LinearLayout.VERTICAL)
  layout.setBackgroundColor(0xFF000000) -- لون أسود

  local titleLayout = LinearLayout(activity)
  titleLayout.setOrientation(LinearLayout.HORIZONTAL) -- تخطيط أفقي
  local titleText = TextView(activity)
  titleText.setText("Contacts")
  titleText.setTextSize(30)
  titleText.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {
    onClick = function(view)
      service.click({{"%返回"}})
    end
  }))
  titleText.setOnLongClickListener(luajava.createProxy("android.view.View$OnLongClickListener", {
    onLongClick = function(view)
      task(200, function()
        getAllCallLogs(resultsList)
      end)
      service.click({{"%返回"}})
      return true -- يجب إرجاع true للإشارة إلى أن الحدث تم التعامل معه
    end
  }))
  local paramsTitle = LinearLayout.LayoutParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1)
  titleText.setLayoutParams(paramsTitle) -- إعداد الوزن ليكون متساوي
  titleLayout.addView(titleText)
  layout.addView(titleLayout) -- إضافة التخطيط للعنوان وزر الإغلاق إلى التخطيط الرئيسي

  local contactCount = TextView(activity)
  contactCount.setText("contact Count: " .. #contactsList)
  contactCount.setTextSize(23)
  layout.addView(contactCount)

  local searchBox = EditText(activity)
  searchBox.contentDescription = ("Search by name or number")
  searchBox.setBackgroundColor(0xFF388E3C) -- لون أخضر داكن
  layout.addView(searchBox)


  local toggleFavoritesButton = Button(activity)
  toggleFavoritesButton.setText("Show Favorites")
  local showingFavorites = false -- متغير للتحقق من حالة العرض الحالية
  local favoriteColor = 0xFFB8860B -- لون ذهبي قاتم (ARGB)
  local allContactsColor = 0xFF00008B -- الأزرق القاتم جداً لجميع جهات الاتصال (ARGB)
  toggleFavoritesButton.setBackgroundColor(allContactsColor)
  toggleFavoritesButton.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {
    onClick = function(view)
      showingFavorites = not showingFavorites
      if showingFavorites then
        toggleFavoritesButton.setText("Show All Contacts")
        toggleFavoritesButton.setBackgroundColor(favoriteColor) -- تعيين اللون الذهبي
        filteredContactsList = getFavoriteContacts(contactsList)
        sortContacts(filteredContactsList) -- إضافة الفرز هنا
       else
        toggleFavoritesButton.setText("Show Favorites")
        toggleFavoritesButton.setBackgroundColor(allContactsColor) -- تعيين اللون الأزرق القاتم
        filteredContactsList = contactsList
        sortContacts(filteredContactsList) -- إضافة الفرز هنا
      end
      updateListView2(filteredContactsList)
      contactCount.setText("Contact Count: " .. #filteredContactsList)
    end
  }))

  layout.addView(toggleFavoritesButton)


  local listView = ListView(activity)
  layout.addView(listView)

  sortContacts(contactsList) -- فرز جهات الاتصال
  local filteredContactsList = contactsList -- المتغير المستخدم للتصفية
  local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, formatContacts2(filteredContactsList))
  listView.setAdapter(adapter)

  local dlg = LuaDialog(activity).setView(layout)
  dlg.show()

  searchBox.addTextChangedListener(luajava.createProxy("android.text.TextWatcher", {
    onTextChanged = function(s, start, before, count)
      local query = tostring(s):lower()
      filteredContactsList = {} -- تصفية القائمة هنا
      for _, contact in ipairs(contactsList) do
        if contact.name:lower():find(query) then
          table.insert(filteredContactsList, contact)
         else
          for _, number in ipairs(contact.numbers) do
            if number:lower():find(query) then
              table.insert(filteredContactsList, contact)
              break
            end
          end
        end
      end
      updateListView2(filteredContactsList)

      contactCount.setText("contact Count: " .. #filteredContactsList)


      if #filteredContactsList > 0 then
        local firstContact = filteredContactsList[1].name
        task(100, function()
          service.appendSpeak2(#filteredContactsList .. firstContact)
        end)
       else
        task(100, function()
          service.appendSpeak2("0")
        end)
      end
    end,
    beforeTextChanged = function(s, start, count, after) end,
    afterTextChanged = function(s) end
  }))


  listView.setOnItemClickListener(luajava.createProxy("android.widget.AdapterView$OnItemClickListener", {
    onItemClick = function(parent, view, position, id)
      local text = tostring(view.getText())
      local lines = {}
      for line in text:gmatch("([^\n]+)") do
        table.insert(lines, line)
      end
      local validLines = {}
      for _, line in ipairs(lines) do
        if line:match("^%+?%d") then
          table.insert(validLines, line)
        end
      end
      if #validLines == 1 then
        local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(validLines[1])))
        activity.startActivity(i)
        dlg.dismiss()
       else
        service.click({{"%返回$0", "%链接"}})
      end
    end
  }))


  listView.setOnItemLongClickListener(luajava.createProxy("android.widget.AdapterView$OnItemLongClickListener", {
    onItemLongClick = function(parent, view, position, id)

      local subMenuItems = {
        "Select SIM card",
        "Send SMS",
        "Show history",
        "Show contact info",
        "Edit contact info",
        "Copy contact info",
        "Share contact info",
      }

      layout = {
        GridView,
        id = "grid",
        numColumns = 2, -- عدد الأعمدة في الشبكة
        background = "#ff00002e", -- لون الخلفية
        layout_width = "fill",
        layout_height = "fill"
      }

      dlg = LuaDialog(service)
      service.postSpeak(200,"Choose an option")
      dlg.View = loadlayout(layout)
      grid.adapter = SingleLineAdapter(service, String(subMenuItems))
      grid.onItemClick = function(_, _, position)
        dlg.dismiss()

        if position == 0 then
          local text = tostring(view.getText())
          local lines = {}
          for line in text:gmatch("[^\r\n]+") do
            table.insert(lines, line)
          end
          local numbers = {}
          for i = 2, #lines do
            table.insert(numbers, lines[i])
          end
          if #numbers == 1 then
            local i = Intent(Intent.ACTION_DIAL, Uri.parse("tel:" .. Uri.encode(numbers[1])))
            activity.startActivity(i)
            service.click({{"%返回"}})
           else
            local numberDialog = LuaDialog(activity)
            local numberAdapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, numbers)
            local numberListView = ListView(activity)
            numberListView.setAdapter(numberAdapter)
            numberListView.setOnItemClickListener(luajava.createProxy("android.widget.AdapterView$OnItemClickListener", {
              onItemClick = function(parent, view, numPosition, id)
                local i = Intent(Intent.ACTION_DIAL, Uri.parse("tel:" .. Uri.encode(numbers[numPosition + 1])))
                activity.startActivity(i)
                numberDialog.dismiss()
                service.click({{"%返回"}})
              end
            }))
            numberDialog.setView(numberListView)
            numberDialog.setTitle("Choose a number to call")
            numberDialog.show()
          end

         elseif position == 1 then
          local text = tostring(view.getText())
          local lines = {}
          for line in text:gmatch("[^\r\n]+") do
            table.insert(lines, line)
          end
          local numbers = {}
          for i = 2, #lines do
            table.insert(numbers, lines[i])
          end
          local presetMessage = "السلام عليكم\n"
          if #numbers == 1 then
            local i = Intent(Intent.ACTION_SENDTO, Uri.parse("smsto:" .. Uri.encode(numbers[1])))
            i.putExtra("sms_body", presetMessage)
            activity.startActivity(i)
            service.click({{"%返回"}})
           else
            local numberDialog = LuaDialog(activity)
            local numberAdapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, numbers)
            local numberListView = ListView(activity)
            numberListView.setAdapter(numberAdapter)
            numberListView.setOnItemClickListener(luajava.createProxy("android.widget.AdapterView$OnItemClickListener", {
              onItemClick = function(parent, view, numPosition, id)
                local i = Intent(Intent.ACTION_SENDTO, Uri.parse("smsto:" .. Uri.encode(numbers[numPosition + 1])))
                i.putExtra("sms_body", presetMessage)
                activity.startActivity(i)
                numberDialog.dismiss()
                service.click({{"%返回"}})
              end
            }))
            numberDialog.setView(numberListView)
            numberDialog.setTitle("Choose a number to message")
            numberDialog.show()
          end

         elseif position == 2 then
          local text = tostring(view.getText()) -- جلب النص الكامل من `view`
          local lines = {}
          for line in text:gmatch("([^\n]*)\n?") do
            table.insert(lines, line)
          end
          if #lines > 0 then
            local firstLine = lines[1]
            firstLine = firstLine:gsub("[.,]%s*$", "")
            service.click({{"%返回"}})
            task(400, function()
              getAllCallLogs(resultsList)
              task(500, function()
                service.setText(firstLine)
              end)
            end)
           else
            this.postSpeak(200, "النص يحتوي على أقل من سطر واحد.")
          end

         elseif position == 3 then
          local text = tostring(view.getText())
          local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول
          contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
          if not contactName then
            Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
            return
          end
          local contentResolver = activity.getContentResolver()
          local cursor = contentResolver.query(
          ContactsContract.Contacts.CONTENT_URI,
          { ContactsContract.Contacts._ID },
          ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
          { contactName },
          nil
          )
          if cursor ~= nil and cursor.moveToFirst() then
            local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            cursor.close()
            local viewIntent = Intent(Intent.ACTION_VIEW)
            viewIntent.setData(Uri.withAppendedPath(ContactsContract.Contacts.CONTENT_URI, contactId))
            activity.startActivity(viewIntent)
            service.click({{"%返回"}})
           else
            Toast.makeText(activity, "Contact not found", Toast.LENGTH_SHORT).show()
          end

         elseif position == 4 then
          local text = tostring(view.getText())

          local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول

          contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
          if not contactName then
            Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
            return
          end
          local contentResolver = activity.getContentResolver()

          local cursor = contentResolver.query(
          ContactsContract.Contacts.CONTENT_URI,
          { ContactsContract.Contacts._ID },
          ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
          { contactName },
          nil
          )
          if cursor ~= nil and cursor.moveToFirst() then
            local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            cursor.close()
            local editIntent = Intent(Intent.ACTION_EDIT)
            editIntent.setData(Uri.withAppendedPath(ContactsContract.Contacts.CONTENT_URI, contactId))
            editIntent.putExtra("finishActivityOnSaveCompleted", true)
            activity.startActivity(editIntent)
            service.click({{"%返回"}})
           else
            Toast.makeText(activity, "Contact not found", Toast.LENGTH_SHORT).show()
          end

         elseif position == 5 then
          local text = tostring(view.getText())
          local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول
          contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
          if not contactName then
            Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
            return
          end
          local contentResolver = activity.getContentResolver()
          local cursor = contentResolver.query(
          ContactsContract.Contacts.CONTENT_URI,
          { ContactsContract.Contacts._ID, ContactsContract.Contacts.HAS_PHONE_NUMBER },
          ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
          { contactName },
          nil
          )
          if cursor ~= nil and cursor.moveToFirst() then
            local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            local hasPhoneNumber = cursor.getInt(cursor.getColumnIndex(ContactsContract.Contacts.HAS_PHONE_NUMBER))
            cursor.close()
            local contactInfo = contactName
            if hasPhoneNumber > 0 then
              local phoneCursor = contentResolver.query(
              ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
              { ContactsContract.CommonDataKinds.Phone.NUMBER },
              ContactsContract.CommonDataKinds.Phone.CONTACT_ID .. " = ?",
              { contactId },
              nil
              )
              local phoneNumbers = {}
              local uniqueNumbers = {} -- تخزين الأرقام الفريدة فقط
              while phoneCursor ~= nil and phoneCursor.moveToNext() do
                local phoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER)):gsub("%s+", "")
                if not uniqueNumbers[phoneNumber] then
                  uniqueNumbers[phoneNumber] = true -- علامة على وجود الرقم
                  table.insert(phoneNumbers, phoneNumber) -- إضافة الرقم بعد التأكد من عدم التكرار
                end
              end
              phoneCursor.close()
              contactInfo = contactInfo .. "\n" .. table.concat(phoneNumbers, "\n")
             else
              contactInfo = contactInfo .. "\nNo phone number found for this contact."
            end
            service.copy(contactInfo)
            Toast.makeText(activity, "Contact info copied to clipboard.", Toast.LENGTH_SHORT).show()
            service.click({{"%返回"}})
           else
            Toast.makeText(activity, "Contact not found.", Toast.LENGTH_SHORT).show()
          end

         elseif position == 6 then
          local text = tostring(view.getText())
          local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول
          contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
          if not contactName then
            Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
            return
          end
          local contentResolver = activity.getContentResolver()
          local cursor = contentResolver.query(
          ContactsContract.Contacts.CONTENT_URI,
          { ContactsContract.Contacts._ID, ContactsContract.Contacts.HAS_PHONE_NUMBER },
          ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
          { contactName },
          nil
          )
          if cursor ~= nil and cursor.moveToFirst() then
            local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            local hasPhoneNumber = cursor.getInt(cursor.getColumnIndex(ContactsContract.Contacts.HAS_PHONE_NUMBER))
            cursor.close()
            local contactInfo = contactName
            if hasPhoneNumber > 0 then
              local phoneCursor = contentResolver.query(
              ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
              { ContactsContract.CommonDataKinds.Phone.NUMBER },
              ContactsContract.CommonDataKinds.Phone.CONTACT_ID .. " = ?",
              { contactId },
              nil
              )
              local phoneNumbers = {}
              local uniqueNumbers = {} -- تخزين الأرقام الفريدة فقط
              while phoneCursor ~= nil and phoneCursor.moveToNext() do
                local phoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER)):gsub("%s+", "")
                if not uniqueNumbers[phoneNumber] then
                  uniqueNumbers[phoneNumber] = true -- علامة على وجود الرقم
                  table.insert(phoneNumbers, phoneNumber) -- إضافة الرقم بعد التأكد من عدم التكرار
                end
              end
              phoneCursor.close()
              contactInfo = contactInfo .. "\n" .. table.concat(phoneNumbers, "\n")
             else
              contactInfo = contactInfo .. "\nNo phone number found for this contact."
            end
            local sendIntent = Intent(Intent.ACTION_SEND)
            sendIntent.setType("text/plain")
            sendIntent.putExtra(Intent.EXTRA_TEXT, contactInfo)
            activity.startActivity(Intent.createChooser(sendIntent, "Send Contact Info"))
            service.click({{"%返回"}})
           else
            Toast.makeText(activity, "Contact not found.", Toast.LENGTH_SHORT).show()
          end


        end
      end

      grid.onItemLongClick = function(_, _, position)
        dlg.dismiss()

        if position == 0 then

         elseif position == 1 then

         elseif position == 2 then

         elseif position == 3 then

         elseif position == 4 then

         elseif position == 5 then

         elseif position == 6 then

        end
      end

      dlg.show()
      return true -- مهم للإشارة إلى أن الضغط المطوّل تم التعامل معه
    end
  }))

  function updateListView2(filteredContacts)
    adapter.clear()
    adapter.addAll(formatContacts2(filteredContacts))
    adapter.notifyDataSetChanged()
  end
end




function formatContacts2(contacts)
  local formatted = {}
  for _, contact in ipairs(contacts) do
    local numbersStr = table.concat(contact.numbers, "\n") -- استخدم "\n" لضم الأرقام في سطور مستقلة
    table.insert(formatted, contact.name .. "\n" .. numbersStr .. "\n") -- إضافة "\n" بعد الأرقام
  end
  return formatted
end



function getFavoriteContacts(contactsList)
  local favoriteContacts = {}
  local activity = this
  local resolver = activity.getContentResolver()
  local cursor = resolver.query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
  {ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME, ContactsContract.CommonDataKinds.Phone.NUMBER},
  ContactsContract.Contacts.STARRED .. " = ?",
  {"1"}, -- 1 تعني مفضل
  nil)

  if cursor then
    while cursor.moveToNext() do
      local name = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME))
      local number = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
      number = normalizeNumber(number)
      if favoriteContacts[name] == nil then
        favoriteContacts[name] = {}
      end
      if not table.contains(favoriteContacts[name], number) then
        table.insert(favoriteContacts[name], number)
      end
    end
    cursor.close()
  end
  local uniqueFavoriteContactsList = {}
  for name, numbers in pairs(favoriteContacts) do
    table.insert(uniqueFavoriteContactsList, {name = name, numbers = numbers})
  end
  return uniqueFavoriteContactsList
end


