function showDailyTasks()
  local file_path = "/storage/emulated/0/Documents/comfortableDictation/calendarData.json"

  local function loadData()
    local list, raw = {}, {}
    local fr = io.open(file_path, "r")
    if fr then
      for line in fr:lines() do
        local date, content = line:match('%["(.-)",%s*"(.-)"%]')
        if date and content and date == os.date("%d-%m-%Y") then
          table.insert(list, {date, content})  -- تخزين التاريخ والمحتوى معًا
          table.insert(raw, line)
        end
      end
      fr:close()
    end
    return list, raw
  end

  local dataList, rawLines = loadData()
  local currentDate = os.date("%d-%m-%Y")

  -- ترتيب المهام حسب التاريخ (من الأقدم إلى الأحدث)
  table.sort(dataList, function(a, b)
    return a[1] < b[1]  -- مقارنة التاريخ
  end)

  local layout = {
    LinearLayout,
    orientation = "vertical",
    padding = "8dp",
    backgroundColor = 0xFF000000,
    {
      TextView,
      id = "headerText", -- إضافة id لعنوان النص
      layout_width = "match_parent",
      layout_height = "wrap_content",
      text = "مهام " .. currentDate,
      textSize = "20sp",
      gravity = "center",
      textColor = 0xFFFFFFFF,
      backgroundColor = 0xFF000000
    },
    {
      ListView,
      id = "listView",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      backgroundColor = 0xFF000000
    },
    {
      LinearLayout,
      orientation = "horizontal",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "مهمة جديدة",
        backgroundColor = 0xFF006400,
        textColor = 0xFFFFFFFF,
        onClick = function()
          service.click({{"%Retour",}})
          task(20, function()
            addNewTask()
          end)
        end
      },
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "خروج",
        backgroundColor = 0xFF8B0000,
        textColor = 0xFFFFFFFF,
        onClick = function()
          service.click({{"%Retour",}})
        end
      }
    },
  }

  local views = {}
  local dlg = LuaDialog(this)
  dlg.setView(loadlayout(layout, views))

  -- فقط المحتوى بدون التاريخ
  local taskContentList = {}
  for _, task in ipairs(dataList) do
    table.insert(taskContentList, task[2])  -- إضافة المحتوى فقط
  end

  local adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, taskContentList)
  views.listView.setAdapter(adapter)

  views.listView.setOnItemClickListener(AdapterView.OnItemClickListener{
    onItemClick = function(parent, view, position, id)
      local oldLine = rawLines[position + 1]
      local oldDate, oldContent = oldLine:match('%["(.-)",%s*"(.-)"%]')
      if not oldDate or not oldContent then return end

      local layout = {
        LinearLayout,
        orientation = "vertical",
        padding = "16dp",
        backgroundColor = 0xFF000000,
        {
          TextView,
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = "تعديل المهمة.",
          textSize = "20sp",
          gravity = "center",
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF000000
        },
        {
          EditText,
          id = "dateInput",
          hint = "أدخل التاريخ",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          inputType = "number",
          text = oldDate,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          EditText,
          id = "contentInput",
          hint = "أدخل المهمة",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = oldContent,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          LinearLayout,
          orientation = "horizontal",
          gravity = "center",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          {
            Button,
            id = "saveButton",
            text = "حفظ",
            layout_weight = "1",
            backgroundColor = 0xFF004D00,
            textColor = 0xFFFFFFFF
          },
          {
            Button,
            id = "cancelButton",
            text = "إلغاء",
            layout_weight = "1",
            backgroundColor = 0xFF8B0000,
            textColor = 0xFFFFFFFF
          }
        }
      }

      local editViews = {}
      local editDlg = LuaDialog(this)
      editDlg.setView(loadlayout(layout, editViews))

      editViews.saveButton.onClick = function()
        local newDate = editViews.dateInput.getText()
        local newContent = editViews.contentInput.getText()
        local newLine = string.format('["%s", "%s"]', newDate, newContent)

        local fw = io.open(file_path, "w")
        if fw then
          for i, line in ipairs(rawLines) do
            if i == position + 1 then
              fw:write(newLine .. "\n")
            else
              fw:write(line .. "\n")
            end
          end
          fw:close()
        end
        -- تحديث القائمة المعروضة
        dataList[position + 1] = {newDate, newContent}
        taskContentList[position + 1] = newContent
        adapter.notifyDataSetChanged()
        editDlg.dismiss()
        Toast.makeText(this, "تم التعديل", Toast.LENGTH_SHORT).show()
      end

      editViews.cancelButton.onClick = function()
        editDlg.dismiss()
      end

      editDlg.show()
    end
  })

  views.listView.setOnItemLongClickListener(AdapterView.OnItemLongClickListener{
    onItemLongClick = function(parent, view, position, id)
      local confirmDialog = LuaDialog(this)
      confirmDialog.setTitle("تأكيد الحذف")
      confirmDialog.setMessage("هل تريد حذف هذه المهمة؟")
      confirmDialog.setPositiveButton("نعم", function()
        table.remove(rawLines, position + 1)
        adapter.remove(adapter.getItem(position))
        local fw = io.open(file_path, "w")
        if fw then
          for _, line in ipairs(rawLines) do
            fw:write(line .. "\n")
          end
          fw:close()
        end
        adapter.notifyDataSetChanged()
        Toast.makeText(this, "تم الحذف", Toast.LENGTH_SHORT).show()
      end)
      confirmDialog.setNegativeButton("إلغاء", function() end)
      confirmDialog.show()
      return true
    end
  })

  -- إضافة وظيفة الضغط المطول على عنوان النص
  views.headerText.setOnLongClickListener(View.OnLongClickListener{
    onLongClick = function(v)
      local confirmDeleteAllDialog = LuaDialog(this)
      confirmDeleteAllDialog.setTitle("تأكيد حذف جميع مهام اليوم")
      confirmDeleteAllDialog.setMessage("هل أنت متأكد أنك تريد حذف جميع مهام اليوم؟")
      confirmDeleteAllDialog.setPositiveButton("نعم", function()
        local today = os.date("%d-%m-%Y")
        local allLines = {}
        local fr = io.open(file_path, "r")
        if fr then
          for line in fr:lines() do
            table.insert(allLines, line)
          end
          fr:close()
        end

        local newLines = {}
        for _, line in ipairs(allLines) do
          local date = line:match('%["(.-)",')
          if date ~= today then
            table.insert(newLines, line .. "\n")
          end
        end

        local fw = io.open(file_path, "w")
        if fw then
          for _, line in ipairs(newLines) do
            fw:write(line)
          end
          fw:close()
        end

        -- إعادة تحميل البيانات وتحديث القائمة
        dataList, rawLines = loadData()
        taskContentList = {}
        for _, task in ipairs(dataList) do
          table.insert(taskContentList, task[2])
        end
        adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, taskContentList)
        views.listView.setAdapter(adapter)
        Toast.makeText(this, "تم حذف جميع مهام اليوم", Toast.LENGTH_SHORT).show()
      end)
      confirmDeleteAllDialog.setNegativeButton("إلغاء", function() end)
      confirmDeleteAllDialog.show()
      return true
    end
  })

  dlg.show()
end











function showWeeklyTasks()
  local file_path = "/storage/emulated/0/Documents/comfortableDictation/calendarData.json"

  local function loadData()
    local list, raw = {}, {}
    local entries = {}
    local fr = io.open(file_path, "r")
    if fr then
      for line in fr:lines() do
        local date, content = line:match('%["(.-)",%s*"(.-)"%]')
        if date and content then
          local day, month, year = date:match("^(%d%d)-(%d%d)-(%d%d%d%d)$")
          if day and month and year then
            local currentDate = os.date("*t")
            local today = os.time{year=currentDate.year, month=currentDate.month, day=currentDate.day}
            local daysSinceSaturday = (currentDate.wday % 7)
            local weekStartTime = today - (daysSinceSaturday * 86400)
            local weekEndTime = weekStartTime + (6 * 86400)

            local taskDate = os.time{year=tonumber(year), month=tonumber(month), day=tonumber(day)}
            if taskDate >= weekStartTime and taskDate <= weekEndTime then
              table.insert(entries, {
                display = date .. " - " .. content,
                raw = line,
                timestamp = taskDate
              })
            end
          end
        end
      end
      fr:close()
    end

    table.sort(entries, function(a, b) return a.timestamp < b.timestamp end)

    for _, entry in ipairs(entries) do
      table.insert(list, entry.display)
      table.insert(raw, entry.raw)
    end

    return list, raw
  end

  local function deleteAllWeeklyTasks()
    local currentDate = os.date("*t")
    local today = os.time{year=currentDate.year, month=currentDate.month, day=currentDate.day}
    local daysSinceSaturday = (currentDate.wday % 7)
    local weekStartTime = today - (daysSinceSaturday * 86400)
    local weekEndTime = weekStartTime + (6 * 86400)
    local newLines = {}
    local changesMade = false

    local fr = io.open(file_path, "r")
    if fr then
      for line in fr:lines() do
        local date = line:match('%["(.-)",')
        if date then
          local day, month, year = date:match("^(%d%d)-(%d%d)-(%d%d%d%d)$")
          if day and month and year then
            local taskDate = os.time{year=tonumber(year), month=tonumber(month), day=tonumber(day)}
            if not (taskDate >= weekStartTime and taskDate <= weekEndTime) then
              table.insert(newLines, line)
            else
              changesMade = true
            end
          else
            table.insert(newLines, line)
          end
        else
          table.insert(newLines, line)
        end
      end
      fr:close()
    end

    local fw = io.open(file_path, "w")
    if fw then
      for _, line in ipairs(newLines) do
        fw:write(line .. "\n")
      end
      fw:close()
    end

    if changesMade then
      Toast.makeText(this, "تم حذف مهام هذا الأسبوع", Toast.LENGTH_SHORT).show()
      service.postSpeak(300, "تم حذف مهام هذا الأسبوع.")
    else
      Toast.makeText(this, "لا توجد مهام لحذفها هذا الأسبوع", Toast.LENGTH_SHORT).show()
      service.postSpeak(300, "لا توجد مهام لحذفها هذا الأسبوع.")
    end
    showWeeklyTasks() -- تحديث عرض المهام بعد الحذف
  end

  local dataList, rawLines = loadData()

  local layout = {
    LinearLayout,
    orientation = "vertical",
    padding = "8dp",
    backgroundColor = 0xFF000000,
    {
      TextView,
      id = "weeklyTasksTitle",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      text = "مهام هذا الأسبوع",
      textSize = "20sp",
      gravity = "center",
      textColor = 0xFFFFFFFF,
      backgroundColor = 0xFF000000
    },
    {
      ListView,
      id = "listView",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      backgroundColor = 0xFF000000
    },
    {
      LinearLayout,
      orientation = "horizontal",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "مهمة جديدة",
        backgroundColor = 0xFF006400,
        textColor = 0xFFFFFFFF,
        onClick = function()
          service.click({{"%Retour",}})
          task(20, function()
            addNewTask()
          end)
        end
      },
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "خروج",
        backgroundColor = 0xFF8B0000,
        textColor = 0xFFFFFFFF,
        onClick = function()
          service.click({{"%Retour",}})
        end
      }
    },
  }

  local views = {}
  local dlg = LuaDialog(this)
  dlg.setView(loadlayout(layout, views))

  local weeklyTasksTitleTextView = views.weeklyTasksTitle
  weeklyTasksTitleTextView.setOnLongClickListener(View.OnLongClickListener{
    onLongClick = function(view)
      local confirmDeleteAllDialog = LuaDialog(this)
      confirmDeleteAllDialog.setTitle("تأكيد الحذف")
      confirmDeleteAllDialog.setMessage("هل أنت متأكد أنك تريد حذف جميع مهام هذا الأسبوع؟")
      confirmDeleteAllDialog.setPositiveButton("نعم", function()
        deleteAllWeeklyTasks()
        confirmDeleteAllDialog.dismiss()
      end)
      confirmDeleteAllDialog.setNegativeButton("إلغاء", function()
        confirmDeleteAllDialog.dismiss()
      end)
      confirmDeleteAllDialog.show()
      return true
    end
  })

  local adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, dataList)
  views.listView.setAdapter(adapter)

  views.listView.setOnItemClickListener(AdapterView.OnItemClickListener{
    onItemClick = function(parent, view, position, id)
      local oldLine = rawLines[position + 1]
      local oldDate, oldContent = oldLine:match('%["(.-)",%s*"(.-)"%]')
      if not oldDate or not oldContent then return end

      local layout = {
        LinearLayout,
        orientation = "vertical",
        padding = "16dp",
        backgroundColor = 0xFF000000,
        {
          TextView,
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = "تعديل المهمة.",
          textSize = "20sp",
          gravity = "center",
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF000000
        },
        {
          EditText,
          id = "dateInput",
          hint = "أدخل التاريخ",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          inputType = "number",
          text = oldDate,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          EditText,
          id = "contentInput",
          hint = "أدخل المهمة",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = oldContent,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          LinearLayout,
          orientation = "horizontal",
          gravity = "center",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          {
            Button,
            id = "saveButton",
            text = "حفظ",
            layout_weight = "1",
            backgroundColor = 0xFF004D00,
            textColor = 0xFFFFFFFF
          },
          {
            Button,
            id = "cancelButton",
            text = "إلغاء",
            layout_weight = "1",
            backgroundColor = 0xFF8B0000,
            textColor = 0xFFFFFFFF
          }
        }
      }

      local editViews = {}
      local editDlg = LuaDialog(this)
      editDlg.setView(loadlayout(layout, editViews))

      editViews.saveButton.setOnClickListener(function()
        local newDate = tostring(editViews.dateInput.getText()):gsub("\n", "")
        local newContent = tostring(editViews.contentInput.getText()):gsub("\n", "")

        if newDate == "" or newContent == "" then
          Toast.makeText(this, "يرجى ملء كلا الحقلين", Toast.LENGTH_SHORT).show()
          service.postSpeak(300, "يرجى إدخال التاريخ والمحتوى أولاً.")
          return
        end

        rawLines[position + 1] = string.format('["%s", "%s"]', newDate, newContent)
        local fw = io.open(file_path, "w")
        if fw then
          for _, line in ipairs(rawLines) do
            fw:write(line .. "\n")
          end
          fw:close()
        end

        adapter.remove(adapter.getItem(position))
        adapter.insert(newDate .. " - " .. newContent, position)
        adapter.notifyDataSetChanged()
        Toast.makeText(this, "تم التعديل", Toast.LENGTH_SHORT).show()
        editDlg.dismiss()
      end)

      editViews.cancelButton.setOnClickListener(function()
        editDlg.dismiss()
      end)

      editDlg.show()
    end
  })

  dlg.show()
end




function showMonthlyTasks()
  local file_path = "/storage/emulated/0/Documents/comfortableDictation/calendarData.json"

  local function loadData()
    local items = {}
    local fr = io.open(file_path, "r")
    if fr then
      local currentYear = os.date("%Y")
      local currentMonth = os.date("%m")
      for line in fr:lines() do
        local date, content = line:match('%["(.-)",%s*"(.-)"%]')
        if date and content then
          local _, m, y = date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
          if y == currentYear and m == currentMonth then
            table.insert(items, {date = date, content = content, raw = line})
          end
        end
      end
      fr:close()
    end

    table.sort(items, function(a, b)
      local d1, m1, y1 = a.date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
      local d2, m2, y2 = b.date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
      local t1 = os.time{day = tonumber(d1), month = tonumber(m1), year = tonumber(y1)}
      local t2 = os.time{day = tonumber(d2), month = tonumber(m2), year = tonumber(y2)}
      return t1 < t2
    end)

    return items
  end

  local months = {
    ["01"] = "جانفي", ["02"] = "فيفري", ["03"] = "مارس",
    ["04"] = "أفريل", ["05"] = "ماي", ["06"] = "جوان",
    ["07"] = "جويلية", ["08"] = "أوت", ["09"] = "سبتمبر",
    ["10"] = "أكتوبر", ["11"] = "نوفمبر", ["12"] = "ديسمبر"
  }
  local currentMonthNumber = os.date("%m")
  local currentMonthName = months[currentMonthNumber] or currentMonthNumber

  local allItems = loadData()
  local dlg = LuaDialog(this)

  local layout = {
    LinearLayout,
    orientation = "vertical",
    padding = "8dp",
    {
      TextView,
      layout_width = "match_parent",
      layout_height = "wrap_content",
      text = "مهام شهر " .. currentMonthName,
      textSize = "20sp",
      gravity = "center",
      textColor = 0xFFFFFFFF,
      backgroundColor = 0xFF000000,
      id = "monthTitle"
    },
    {
      EditText,
      id = "searchBox",
      hint = "ابحث في المهام",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      textColor = 0xFFFFFFFF,
      backgroundColor = 0xFF0D47A1
    },
    {
      ListView,
      id = "listView",
      layout_width = "match_parent",
      layout_height = "0dp",
      layout_weight = 1,
      backgroundColor = 0xFF000000
    },
    {
      LinearLayout,
      orientation = "horizontal",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "مهمة جديدة",
        backgroundColor = 0xFF006400,
        textColor = 0xFFFFFFFF,
        onClick = function()
          dlg.dismiss()
          addNewTask()
        end
      },
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "خروج",
        backgroundColor = 0xFF8B0000,
        textColor = 0xFFFFFFFF,
        onClick = function()
          dlg.dismiss()
        end
      }
    }
  }

  local views = {}
  dlg.setView(loadlayout(layout, views))

  local adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, {})
  views.listView.setAdapter(adapter)

  local filteredItems = {}

  local function updateList(filterText)
    adapter.clear()
    filteredItems = {}
    for _, item in ipairs(allItems) do
      local displayText = item.date .. "\n" .. item.content
      if displayText:lower():find(filterText:lower(), 1, true) then
        adapter.add(displayText)
        table.insert(filteredItems, item)
      end
    end
    adapter.notifyDataSetChanged()
  end

  updateList("")

  -- وظيفة حذف جميع مهام الشهر الحالي
  views.monthTitle.setOnLongClickListener(View.OnLongClickListener{
    onLongClick = function(v)
      local confirmDeleteAllDialog = LuaDialog(this)
      confirmDeleteAllDialog.setTitle("تأكيد الحذف")
      confirmDeleteAllDialog.setMessage("هل تريد حذف جميع مهام شهر "..currentMonthName.." "..os.date("%Y").."؟")
      confirmDeleteAllDialog.setPositiveButton("نعم", function()
        local currentYear = os.date("%Y")
        local currentMonth = os.date("%m")
        local linesToKeep = {}
        local deletedCount = 0
        
        -- قراءة الملف وحفظ ما نريد الاحتفاظ به
        local fr = io.open(file_path, "r")
        if fr then
          for line in fr:lines() do
            local date = line:match('%["(.-)",')
            if date then
              local day, m, y = date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
              if not (y == currentYear and m == currentMonth) then
                table.insert(linesToKeep, line)
              else
                deletedCount = deletedCount + 1
              end
            end
          end
          fr:close()
        end
        
        if deletedCount > 0 then
          -- كتابة الملف مع المحتوى المحدث
          local fw = io.open(file_path, "w")
          if fw then
            for _, line in ipairs(linesToKeep) do
              fw:write(line .. "\n")
            end
            fw:close()
          end
          
          -- إعادة تحميل البيانات وتحديث الواجهة
          allItems = loadData()
          updateList(tostring(views.searchBox.getText()))
          
          Toast.makeText(this, "تم حذف "..deletedCount.." مهمة", Toast.LENGTH_SHORT).show()
        else
          Toast.makeText(this, "لا توجد مهام للحذف", Toast.LENGTH_SHORT).show()
        end
      end)
      confirmDeleteAllDialog.setNegativeButton("إلغاء", nil)
      confirmDeleteAllDialog.show()
      return true
    end
  })

  views.searchBox.addTextChangedListener {
    afterTextChanged = function(s)
      updateList(tostring(s))
    end
  }

  views.listView.setOnItemClickListener(AdapterView.OnItemClickListener{
    onItemClick = function(parent, view, position, id)
      local item = filteredItems[position + 1]
      if not item then return end
      local oldDate, oldContent = item.date, item.content

      local editLayout = {
        LinearLayout,
        orientation = "vertical",
        padding = "16dp",
        backgroundColor = 0xFF000000,
        {
          TextView,
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = "تعديل المهمة.",
          textSize = "20sp",
          gravity = "center",
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF000000
        },
        {
          EditText,
          id = "dateInput",
          hint = "أدخل التاريخ",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          inputType = "number",
          text = oldDate,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          EditText,
          id = "contentInput",
          hint = "أدخل المحتوى",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = oldContent,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          LinearLayout,
          orientation = "horizontal",
          gravity = "center",
          layout_width = "match_parent",
          {
            Button,
            id = "saveButton",
            text = "حفظ",
            layout_weight = 1,
            layout_width = "0dp",
            layout_marginRight = "4dp",
            textColor = 0xFFFFFFFF,
            backgroundColor = 0xFF006400,
            textSize = "16sp"
          },
          {
            Button,
            id = "cancelButton",
            text = "إلغاء",
            layout_weight = 1,
            layout_width = "0dp",
            layout_marginLeft = "4dp",
            textColor = 0xFFFFFFFF,
            backgroundColor = 0xFF8B0000,
            textSize = "16sp"
          }
        }
      }

      local editViews = {}
      local editDlg = LuaDialog(this)
      editDlg.setView(loadlayout(editLayout, editViews))

      local isFormatting = false
      editViews.dateInput.addTextChangedListener{
        afterTextChanged = function(s)
          if isFormatting then return end
          isFormatting = true
          local raw = tostring(s):gsub("[^%d]", "")
          local formatted = ""
          if #raw >= 2 then formatted = raw:sub(1, 2) else formatted = raw end
          if #raw > 2 then formatted = formatted .. "-" .. raw:sub(3, math.min(4, #raw)) end
          if #raw > 4 then formatted = formatted .. "-" .. raw:sub(5, math.min(8, #raw)) end
          if formatted ~= tostring(s) then
            editViews.dateInput.setText(formatted)
            editViews.dateInput.setSelection(#formatted)
          end
          isFormatting = false
        end
      }

      editViews.saveButton.setOnClickListener(function()
        local newDate = tostring(editViews.dateInput.getText()):gsub("\n", "")
        local newContent = tostring(editViews.contentInput.getText()):gsub("\n", "")

        if newDate == "" or newContent == "" then
          Toast.makeText(this, "يرجى ملء كلا الحقلين", Toast.LENGTH_SHORT).show()
          service.postSpeak(300, "يرجى إدخال التاريخ والمحتوى أولاً.")
          return
        end

        local d, m, y = newDate:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
        if not (d and m and y) then
          Toast.makeText(this, "تاريخ غير صالح", Toast.LENGTH_SHORT).show()
          service.speak("تاريخ غير صالح. الرجاء التأكد من اليوم والشهر.")
          return
        end

        item.date = newDate
        item.content = newContent
        item.raw = string.format('["%s", "%s"]', newDate, newContent)

        local fw = io.open(file_path, "w")
        if fw then
          for _, i in ipairs(allItems) do
            fw:write(i.raw .. "\n")
          end
          fw:close()
        end

        updateList(tostring(views.searchBox.getText()))
        Toast.makeText(this, "تم التعديل", Toast.LENGTH_SHORT).show()
        editDlg.dismiss()
      end)

      editViews.cancelButton.setOnClickListener(function()
        editDlg.dismiss()
      end)

      editDlg.show()
    end
  })

  views.listView.setOnItemLongClickListener(AdapterView.OnItemLongClickListener{
    onItemLongClick = function(parent, view, position, id)
      local item = filteredItems[position + 1]
      if not item then return true end

      local confirmDialog = LuaDialog(this)
      confirmDialog.setTitle("تأكيد الحذف")
      confirmDialog.setMessage("هل تريد حذف هذا الإدخال؟")
      confirmDialog.setPositiveButton("نعم", function()
        for i = #allItems, 1, -1 do
          if allItems[i] == item then
            table.remove(allItems, i)
            break
          end
        end

        local fw = io.open(file_path, "w")
        if fw then
          for _, i in ipairs(allItems) do
            fw:write(i.raw .. "\n")
          end
          fw:close()
        end

        updateList(tostring(views.searchBox.getText()))
        Toast.makeText(this, "تم الحذف", Toast.LENGTH_SHORT).show()
      end)
      confirmDialog.setNegativeButton("إلغاء", function() end)
      confirmDialog.show()
      return true
    end
  })

  dlg.show()
end


















function showYearlyTasks()
  local file_path = "/storage/emulated/0/Documents/comfortableDictation/calendarData.json"

  local function loadData()
    local items = {}
    local fr = io.open(file_path, "r")
    if fr then
      local currentYear = os.date("%Y")  -- السنة الحالية
      for line in fr:lines() do
        local date, content = line:match('%["(.-)",%s*"(.-)"%]')
        if date and content then
          local _, _, y = date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
          if y == currentYear then
            table.insert(items, {date = date, content = content, raw = line})
          end
        end
      end
      fr:close()
    end

    table.sort(items, function(a, b)
      local d1, m1, y1 = a.date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
      local d2, m2, y2 = b.date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
      local t1 = os.time{day = tonumber(d1), month = tonumber(m1), year = tonumber(y1)}
      local t2 = os.time{day = tonumber(d2), month = tonumber(m2), year = tonumber(y2)}
      return t1 < t2
    end)

    return items
  end

  local allItems = loadData()
  local dlg = LuaDialog(this)

  local layout = {
    LinearLayout,
    orientation = "vertical",
    padding = "8dp",
    {
      TextView,
      layout_width = "match_parent",
      layout_height = "wrap_content",
      text = "مهام السنة الحالية",
      textSize = "20sp",
      gravity = "center",
      textColor = 0xFFFFFFFF,
      backgroundColor = 0xFF000000,
      id = "yearTitle"  -- أضفنا id للعنوان الرئيسي
    },
    {
      EditText,
      id = "searchBox",
      hint = "ابحث في المهام",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      textColor = 0xFFFFFFFF,
      backgroundColor = 0xFF0D47A1
    },
    {
      ListView,
      id = "listView",
      layout_width = "match_parent",
      layout_height = "0dp",
      layout_weight = 1,
      backgroundColor = 0xFF000000
    },
    {
      LinearLayout,
      orientation = "horizontal",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "مهمة جديدة",
        backgroundColor = 0xFF006400,
        textColor = 0xFFFFFFFF,
        onClick = function()
          dlg.dismiss()
          addNewTask()
        end
      },
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "خروج",
        backgroundColor = 0xFF8B0000,
        textColor = 0xFFFFFFFF,
        onClick = function()
          dlg.dismiss()
        end
      }
    }
  }

  local views = {}
  dlg.setView(loadlayout(layout, views))

  local adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, {})
  views.listView.setAdapter(adapter)

  local filteredItems = {}

  local function updateList(filterText)
    adapter.clear()
    filteredItems = {}
    for _, item in ipairs(allItems) do
      local displayText = item.date .. "\n" .. item.content
      if displayText:lower():find(filterText:lower(), 1, true) then
        adapter.add(displayText)
        table.insert(filteredItems, item)
      end
    end
    adapter.notifyDataSetChanged()
  end

  updateList("")

  -- إضافة ميزة الحذف بالضغط المطول على العنوان
  views.yearTitle.setOnLongClickListener(View.OnLongClickListener{
    onLongClick = function(v)
      local confirmDeleteDialog = LuaDialog(this)
      confirmDeleteDialog.setTitle("تأكيد الحذف")
      confirmDeleteDialog.setMessage("هل تريد حذف جميع مهام السنة الحالية؟")
      confirmDeleteDialog.setPositiveButton("نعم", function()
        local currentYear = os.date("%Y")
        local linesToKeep = {}
        local deletedCount = 0
        
        -- قراءة الملف وحفظ ما نريد الاحتفاظ به
        local fr = io.open(file_path, "r")
        if fr then
          for line in fr:lines() do
            local date = line:match('%["(.-)",')
            if date then
              local _, _, y = date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
              if y ~= currentYear then
                table.insert(linesToKeep, line)
              else
                deletedCount = deletedCount + 1
              end
            end
          end
          fr:close()
        end
        
        if deletedCount > 0 then
          -- كتابة الملف مع المحتوى المحدث
          local fw = io.open(file_path, "w")
          if fw then
            for _, line in ipairs(linesToKeep) do
              fw:write(line .. "\n")
            end
            fw:close()
          end
          
          -- إعادة تحميل البيانات وتحديث الواجهة
          allItems = loadData()
          updateList(tostring(views.searchBox.getText()))
          
          Toast.makeText(this, "تم حذف "..deletedCount.." مهمة", Toast.LENGTH_SHORT).show()
        else
          Toast.makeText(this, "لا توجد مهام للحذف", Toast.LENGTH_SHORT).show()
        end
      end)
      confirmDeleteDialog.setNegativeButton("إلغاء", nil)
      confirmDeleteDialog.show()
      return true
    end
  })

  views.searchBox.addTextChangedListener {
    afterTextChanged = function(s)
      updateList(tostring(s))
    end
  }

  views.listView.setOnItemClickListener(AdapterView.OnItemClickListener{
    onItemClick = function(parent, view, position, id)
      local item = filteredItems[position + 1]
      if not item then return end
      local oldDate, oldContent = item.date, item.content

      local editLayout = {
        LinearLayout,
        orientation = "vertical",
        padding = "16dp",
        backgroundColor = 0xFF000000,
        {
          TextView,
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = "تعديل المهمة.",
          textSize = "20sp",
          gravity = "center",
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF000000
        },
        {
          EditText,
          id = "dateInput",
          hint = "أدخل التاريخ",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          inputType = "number",
          text = oldDate,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          EditText,
          id = "contentInput",
          hint = "أدخل المحتوى",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = oldContent,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          LinearLayout,
          orientation = "horizontal",
          gravity = "center",
          layout_width = "match_parent",
          {
            Button,
            id = "saveButton",
            text = "حفظ",
            layout_weight = 1,
            layout_width = "0dp",
            layout_marginRight = "4dp",
            textColor = 0xFFFFFFFF,
            backgroundColor = 0xFF006400,
            textSize = "16sp"
          },
          {
            Button,
            id = "cancelButton",
            text = "إلغاء",
            layout_weight = 1,
            layout_width = "0dp",
            layout_marginLeft = "4dp",
            textColor = 0xFFFFFFFF,
            backgroundColor = 0xFF8B0000,
            textSize = "16sp"
          }
        }
      }

      local editViews = {}
      local editDlg = LuaDialog(this)
      editDlg.setView(loadlayout(editLayout, editViews))

      local isFormatting = false
      editViews.dateInput.addTextChangedListener{
        afterTextChanged = function(s)
          if isFormatting then return end
          isFormatting = true
          local raw = tostring(s):gsub("[^%d]", "")
          local formatted = ""
          if #raw >= 2 then formatted = raw:sub(1, 2) else formatted = raw end
          if #raw > 2 then formatted = formatted .. "-" .. raw:sub(3, math.min(4, #raw)) end
          if #raw > 4 then formatted = formatted .. "-" .. raw:sub(5, math.min(8, #raw)) end
          if formatted ~= tostring(s) then
            editViews.dateInput.setText(formatted)
            editViews.dateInput.setSelection(#formatted)
          end
          isFormatting = false
        end
      }

      editViews.saveButton.setOnClickListener(function()
        local newDate = tostring(editViews.dateInput.getText()):gsub("\n", "")
        local newContent = tostring(editViews.contentInput.getText()):gsub("\n", "")

        if newDate == "" or newContent == "" then
          Toast.makeText(this, "يرجى ملء كلا الحقلين", Toast.LENGTH_SHORT).show()
          service.postSpeak(300, "يرجى إدخال التاريخ والمحتوى أولاً.")
          return
        end

        local d, m, y = newDate:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
        if not (d and m and y) then
          Toast.makeText(this, "تاريخ غير صالح", Toast.LENGTH_SHORT).show()
          service.speak("تاريخ غير صالح. الرجاء التأكد من اليوم والشهر.")
          return
        end

        item.date = newDate
        item.content = newContent
        item.raw = string.format('["%s", "%s"]', newDate, newContent)

        local fw = io.open(file_path, "w")
        if fw then
          for _, i in ipairs(allItems) do
            fw:write(i.raw .. "\n")
          end
          fw:close()
        end

        updateList(tostring(views.searchBox.getText()))
        Toast.makeText(this, "تم التعديل", Toast.LENGTH_SHORT).show()
        editDlg.dismiss()
      end)

      editViews.cancelButton.setOnClickListener(function()
        editDlg.dismiss()
      end)

      editDlg.show()
    end
  })

  views.listView.setOnItemLongClickListener(AdapterView.OnItemLongClickListener{
    onItemLongClick = function(parent, view, position, id)
      local item = filteredItems[position + 1]
      if not item then return true end

      local confirmDialog = LuaDialog(this)
      confirmDialog.setTitle("تأكيد الحذف")
      confirmDialog.setMessage("هل تريد حذف هذا الإدخال؟")
      confirmDialog.setPositiveButton("نعم", function()
        for i = #allItems, 1, -1 do
          if allItems[i] == item then
            table.remove(allItems, i)
            break
          end
        end

        local fw = io.open(file_path, "w")
        if fw then
          for _, i in ipairs(allItems) do
            fw:write(i.raw .. "\n")
          end
          fw:close()
        end

        updateList(tostring(views.searchBox.getText()))
        Toast.makeText(this, "تم الحذف", Toast.LENGTH_SHORT).show()
      end)
      confirmDialog.setNegativeButton("إلغاء", function() end)
      confirmDialog.show()
      return true
    end
  })

  dlg.show()
end








function showAllTasks()
  local file_path = "/storage/emulated/0/Documents/comfortableDictation/calendarData.json"

  local function loadData()
    local items = {}
    local fr = io.open(file_path, "r")
    if fr then
      for line in fr:lines() do
        local date, content = line:match('%["(.-)",%s*"(.-)"%]')
        if date and content then
          table.insert(items, {date = date, content = content, raw = line})
        end
      end
      fr:close()
    end

    table.sort(items, function(a, b)
      local d1, m1, y1 = a.date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
      local d2, m2, y2 = b.date:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
      local t1 = os.time{day = tonumber(d1), month = tonumber(m1), year = tonumber(y1)}
      local t2 = os.time{day = tonumber(d2), month = tonumber(m2), year = tonumber(y2)}
      return t1 < t2
    end)

    return items
  end

  local allItems = loadData()
  local dlg = LuaDialog(this)

  local layout = {
    LinearLayout,
    orientation = "vertical",
    padding = "8dp",
    {
      TextView,
      layout_width = "match_parent",
      layout_height = "wrap_content",
      text = "جميع المهام ",
      textSize = "20sp",
      gravity = "center",
      textColor = 0xFFFFFFFF,
      backgroundColor = 0xFF000000
    },
    {
      EditText,
      id = "searchBox",
      hint = "ابحث في المهام",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      textColor = 0xFFFFFFFF,
      backgroundColor = 0xFF0D47A1
    },
    {
      ListView,
      id = "listView",
      layout_width = "match_parent",
      layout_height = "0dp",
      layout_weight = 1,
      backgroundColor = 0xFF000000
    },
    {
      LinearLayout,
      orientation = "horizontal",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "مهمة جديدة",
        backgroundColor = 0xFF006400,
        textColor = 0xFFFFFFFF,
        onClick = function()
          dlg.dismiss()
          addNewTask()
        end
      },
      {
        Button,
        layout_weight = 1,
        layout_width = "0dp",
        text = "خروج",
        backgroundColor = 0xFF8B0000,
        textColor = 0xFFFFFFFF,
        onClick = function()
          dlg.dismiss()
        end
      }
    }
  }

  local views = {}
  dlg.setView(loadlayout(layout, views))

  local adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, {})
  views.listView.setAdapter(adapter)

  local filteredItems = {}

  local function updateList(filterText)
    adapter.clear()
    filteredItems = {}
    for _, item in ipairs(allItems) do
      local displayText = item.date .. "\n" .. item.content
      if displayText:lower():find(filterText:lower(), 1, true) then
        adapter.add(displayText)
        table.insert(filteredItems, item)
      end
    end
    adapter.notifyDataSetChanged()
  end

  -- عرض الكل في البداية
  updateList("")

  views.searchBox.addTextChangedListener {
    afterTextChanged = function(s)
      updateList(tostring(s))
    end
  }

  views.listView.setOnItemClickListener(AdapterView.OnItemClickListener{
    onItemClick = function(parent, view, position, id)
      local item = filteredItems[position + 1]
      if not item then return end
      local oldDate, oldContent = item.date, item.content

      -- نفس نافذة التعديل كما في كودك السابق
      local editLayout = {
        LinearLayout,
        orientation = "vertical",
        padding = "16dp",
        backgroundColor = 0xFF000000,
        {
          TextView,
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = "تعديل المهمة.",
          textSize = "20sp",
          gravity = "center",
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF000000
        },
        {
          EditText,
          id = "dateInput",
          hint = "أدخل التاريخ",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          inputType = "number",
          text = oldDate,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          EditText,
          id = "contentInput",
          hint = "أدخل المحتوى",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = oldContent,
          textColor = 0xFFFFFFFF,
          backgroundColor = 0xFF0D47A1
        },
        {
          LinearLayout,
          orientation = "horizontal",
          gravity = "center",
          layout_width = "match_parent",
          {
            Button,
            id = "saveButton",
            text = "حفظ",
            layout_weight = 1,
            layout_width = "0dp",
            layout_marginRight = "4dp",
            textColor = 0xFFFFFFFF,
            backgroundColor = 0xFF006400,
            textSize = "16sp"
          },
          {
            Button,
            id = "cancelButton",
            text = "إلغاء",
            layout_weight = 1,
            layout_width = "0dp",
            layout_marginLeft = "4dp",
            textColor = 0xFFFFFFFF,
            backgroundColor = 0xFF8B0000,
            textSize = "16sp"
          }
        }
      }

      local editViews = {}
      local editDlg = LuaDialog(this)
      editDlg.setView(loadlayout(editLayout, editViews))

      local isFormatting = false
      editViews.dateInput.addTextChangedListener{
        afterTextChanged = function(s)
          if isFormatting then return end
          isFormatting = true
          local raw = tostring(s):gsub("[^%d]", "")
          local formatted = ""
          if #raw >= 2 then formatted = raw:sub(1, 2) else formatted = raw end
          if #raw > 2 then formatted = formatted .. "-" .. raw:sub(3, math.min(4, #raw)) end
          if #raw > 4 then formatted = formatted .. "-" .. raw:sub(5, math.min(8, #raw)) end
          if formatted ~= tostring(s) then
            editViews.dateInput.setText(formatted)
            editViews.dateInput.setSelection(#formatted)
          end
          isFormatting = false
        end
      }

      editViews.saveButton.setOnClickListener(function()
        local newDate = tostring(editViews.dateInput.getText()):gsub("\n", "")
        local newContent = tostring(editViews.contentInput.getText()):gsub("\n", "")

        if newDate == "" or newContent == "" then
          Toast.makeText(this, "يرجى ملء كلا الحقلين", Toast.LENGTH_SHORT).show()
          service.postSpeak(300, "يرجى إدخال التاريخ والمحتوى أولاً.")
          return
        end

        local d, m, y = newDate:match("^(%d%d)%-(%d%d)%-(%d%d%d%d)$")
        if not (d and m and y) then
          Toast.makeText(this, "تاريخ غير صالح", Toast.LENGTH_SHORT).show()
          service.speak("تاريخ غير صالح. الرجاء التأكد من اليوم والشهر.")
          return
        end

        item.date = newDate
        item.content = newContent
        item.raw = string.format('["%s", "%s"]', newDate, newContent)

        local fw = io.open(file_path, "w")
        if fw then
          for _, i in ipairs(allItems) do
            fw:write(i.raw .. "\n")
          end
          fw:close()
        end

        updateList(tostring(views.searchBox.getText()))
        Toast.makeText(this, "تم التعديل", Toast.LENGTH_SHORT).show()
        editDlg.dismiss()
      end)

      editViews.cancelButton.setOnClickListener(function()
        editDlg.dismiss()
      end)

      editDlg.show()
    end
  })

  views.listView.setOnItemLongClickListener(AdapterView.OnItemLongClickListener{
    onItemLongClick = function(parent, view, position, id)
      local item = filteredItems[position + 1]
      if not item then return true end

      local confirmDialog = LuaDialog(this)
      confirmDialog.setTitle("تأكيد الحذف")
      confirmDialog.setMessage("هل تريد حذف هذا الإدخال؟")
      confirmDialog.setPositiveButton("نعم", function()
        for i = #allItems, 1, -1 do
          if allItems[i] == item then
            table.remove(allItems, i)
            break
          end
        end

        local fw = io.open(file_path, "w")
        if fw then
          for _, i in ipairs(allItems) do
            fw:write(i.raw .. "\n")
          end
          fw:close()
        end

        updateList(tostring(views.searchBox.getText()))
        Toast.makeText(this, "تم الحذف", Toast.LENGTH_SHORT).show()
      end)
      confirmDialog.setNegativeButton("إلغاء", function() end)
      confirmDialog.show()
      return true
    end
  })

  dlg.show()
end
























































function addNewTask()
  local layout = {
    LinearLayout,
    orientation = "vertical",
    padding = "16dp",
    {
      TextView,
      layout_width = "match_parent",
      layout_height = "wrap_content",
      text = "إدخال البيانات.",
      textSize = "20sp",
      gravity = "center",
      textColor = 0xFFFFFFFF,
      backgroundColor = 0xFF000000
    },
    {
      EditText,
      id = "dateInput",
      hint = "أدخل التاريخ (مثلاً: 05-05-2025)",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      inputType = "number",
      backgroundColor = 0xFF0D47A1
    },
    {
      EditText,
      id = "contentInput",
      hint = "أدخل المحتوى",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      backgroundColor = 0xFF0D47A1
    },
    {
      LinearLayout,
      orientation = "horizontal",
      gravity = "center",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      {
        Button,
        id = "saveButton",
        text = "حفظ",
        layout_weight = "1",
        backgroundColor = 0xFF004D00,
        textColor = 0xFFFFFFFF
      },
      {
        Button,
        id = "cancelButton",
        text = "إلغاء",
        layout_weight = "1",
        backgroundColor = 0xFF8B0000,
        textColor = 0xFFFFFFFF
      }
    }
  }

  local views = {}
  local dlg = LuaDialog(this)
  dlg.setView(loadlayout(layout, views))

  local isFormatting = false
  views.dateInput.addTextChangedListener{
    afterTextChanged = function(s)
      if isFormatting then return end
      isFormatting = true

      local raw = tostring(s):gsub("[^%d]", "")
      local formatted = ""

      if #raw >= 2 then
        formatted = raw:sub(1, 2)
      else
        formatted = raw
      end

      if #raw > 2 then
        formatted = formatted .. "-" .. raw:sub(3, math.min(4, #raw))
      end

      if #raw > 4 then
        formatted = formatted .. "-" .. raw:sub(5, math.min(8, #raw))
      end

      if formatted ~= tostring(s) then
        views.dateInput.setText(formatted)
        views.dateInput.setSelection(#formatted)
      end

      if not formatted:match("^%d%d%-%d%d%-%d%d%d%d$") and #formatted == 10 then
        service.postSpeak(300, "تنسيق التاريخ غير صحيح. الرجاء استخدام الصيغة يوم ناقص شهر ناقص سنة.")
      end

      isFormatting = false
    end
  }

  views.saveButton.setOnClickListener(function()
    local date = tostring(views.dateInput.getText()):gsub("\n", "")
    local content = tostring(views.contentInput.getText()):gsub("\n", "")

    if date == "" or content == "" then
      Toast.makeText(this, "يرجى ملء كلا الحقلين", Toast.LENGTH_SHORT).show()
      service.postSpeak(300, "يرجى إدخال التاريخ والمحتوى أولاً.")
      return
    end

    if not date:match("^%d%d%-%d%d%-%d%d%d%d$") then
      Toast.makeText(this, "تنسيق التاريخ غير صحيح", Toast.LENGTH_SHORT).show()
      service.postSpeak(300, "تنسيق التاريخ غير صحيح. الرجاء استخدام الصيغة يوم ناقص شهر ناقص سنة.")
      return
    end

    local day, month, year = date:match("(%d%d)%-(%d%d)%-(%d%d%d%d)")
    day, month, year = tonumber(day), tonumber(month), tonumber(year)

    local function isValidDate(d, m, y)
      if not (d and m and y) then return false end
      if m < 1 or m > 12 or d < 1 then return false end
      local daysInMonth = {31, (y%4==0 and y%100~=0 or y%400==0) and 29 or 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
      return d <= daysInMonth[m]
    end

    if not isValidDate(day, month, year) then
      Toast.makeText(this, "تاريخ غير صالح", Toast.LENGTH_SHORT).show()
      service.postSpeak(300, "التاريخ غير صالح، تأكد من اليوم والشهر.")
      return
    end

    local file_path = "/storage/emulated/0/Documents/comfortableDictation/calendarData.json"
    local jsonLine = string.format('["%s", "%s"]\n', date, content)

    local fw = io.open(file_path, "a")
    if fw then
      fw:write(jsonLine)
      fw:close()
      service.postSpeak(300, "تم الحفظ بنجاح")
    else
      Toast.makeText(this, "تعذر فتح الملف للحفظ", Toast.LENGTH_SHORT).show()
      service.postSpeak(300, "تعذر حفظ البيانات")
    end
    dlg.dismiss()
  end)

  views.cancelButton.setOnClickListener(function()
    dlg.dismiss()
  end)

  dlg.show()
end












function deleteAllTasks()
      local confirmDeleteAllDialog = LuaDialog(this)
      confirmDeleteAllDialog.setTitle("تأكيد حذف جميع المهام ")
      confirmDeleteAllDialog.setMessage("هل أنت متأكد أنك تريد حذف جميع المهام؟")
      confirmDeleteAllDialog.setPositiveButton("نعم", function()
  local file_path = "/storage/emulated/0/Documents/comfortableDictation/calendarData.json"
  local file = io.open(file_path, "w") -- نفتح الملف بوضع "كتابة" لمسح محتواه
  if file then
    file:close()
    this.postSpeak(200, "تم حذف جميع البيانات.")
  else
    this.postSpeak(200, "تعذر فتح الملف.")
  end
      end)
      confirmDeleteAllDialog.setNegativeButton("إلغاء", function() end)
      confirmDeleteAllDialog.show()
end