function viewContactFromText(activity, node)
  local text = service.getText(node)
  local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول
  contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
  if not contactName then
    Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
    return
  end
  local contentResolver = activity.getContentResolver()
  local cursor = contentResolver.query(
  ContactsContract.Contacts.CONTENT_URI,
  { ContactsContract.Contacts._ID },
  ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
  { contactName },
  nil
  )
  if cursor ~= nil and cursor.moveToFirst() then
    local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
    cursor.close()
    local viewIntent = Intent(Intent.ACTION_VIEW)
    viewIntent.setData(Uri.withAppendedPath(ContactsContract.Contacts.CONTENT_URI, contactId))
    activity.startActivity(viewIntent)
    service.click({{"%返回"}})
   else
    Toast.makeText(activity, "Contact not found", Toast.LENGTH_SHORT).show()
  end

end



function sendContactInfo(activity, node)
  local text = service.getText(node)
  local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول
  contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
  if not contactName then
    Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
    return
  end
  local contentResolver = activity.getContentResolver()
  local cursor = contentResolver.query(
  ContactsContract.Contacts.CONTENT_URI,
  { ContactsContract.Contacts._ID, ContactsContract.Contacts.HAS_PHONE_NUMBER },
  ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
  { contactName },
  nil
  )
  if cursor ~= nil and cursor.moveToFirst() then
    local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
    local hasPhoneNumber = cursor.getInt(cursor.getColumnIndex(ContactsContract.Contacts.HAS_PHONE_NUMBER))
    cursor.close()
    local contactInfo = contactName
    if hasPhoneNumber > 0 then
      local phoneCursor = contentResolver.query(
      ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
      { ContactsContract.CommonDataKinds.Phone.NUMBER },
      ContactsContract.CommonDataKinds.Phone.CONTACT_ID .. " = ?",
      { contactId },
      nil
      )
      local phoneNumbers = {}
      local uniqueNumbers = {} -- تخزين الأرقام الفريدة فقط
      while phoneCursor ~= nil and phoneCursor.moveToNext() do
        local phoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER)):gsub("%s+", "")
        if not uniqueNumbers[phoneNumber] then
          uniqueNumbers[phoneNumber] = true -- علامة على وجود الرقم
          table.insert(phoneNumbers, phoneNumber) -- إضافة الرقم بعد التأكد من عدم التكرار
        end
      end
      phoneCursor.close()
      contactInfo = contactInfo .. "\n" .. table.concat(phoneNumbers, "\n")
     else
      contactInfo = contactInfo .. "\nNo phone number found for this contact."
    end

    local sendIntent = Intent(Intent.ACTION_SEND)
    sendIntent.setType("text/plain")
    sendIntent.putExtra(Intent.EXTRA_TEXT, contactInfo)
    activity.startActivity(Intent.createChooser(sendIntent, "Send Contact Info"))
    service.click({{"%返回"}})
   else
    Toast.makeText(activity, "Contact not found.", Toast.LENGTH_SHORT).show()
  end
end



function copyContactInfo(activity, node)
  local text = service.getText(node)
  local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول

  contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
  if not contactName then
    Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
    return
  end
  local contentResolver = activity.getContentResolver()
  local cursor = contentResolver.query(
  ContactsContract.Contacts.CONTENT_URI,
  { ContactsContract.Contacts._ID, ContactsContract.Contacts.HAS_PHONE_NUMBER },
  ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
  { contactName },
  nil
  )
  if cursor ~= nil and cursor.moveToFirst() then
    local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
    local hasPhoneNumber = cursor.getInt(cursor.getColumnIndex(ContactsContract.Contacts.HAS_PHONE_NUMBER))
    cursor.close()
    local contactInfo = contactName
    if hasPhoneNumber > 0 then
      local phoneCursor = contentResolver.query(
      ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
      { ContactsContract.CommonDataKinds.Phone.NUMBER },
      ContactsContract.CommonDataKinds.Phone.CONTACT_ID .. " = ?",
      { contactId },
      nil
      )
      local phoneNumbers = {}
      local uniqueNumbers = {} -- تخزين الأرقام الفريدة فقط
      while phoneCursor ~= nil and phoneCursor.moveToNext() do
        local phoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER)):gsub("%s+", "")
        if not uniqueNumbers[phoneNumber] then
          uniqueNumbers[phoneNumber] = true -- علامة على وجود الرقم
          table.insert(phoneNumbers, phoneNumber) -- إضافة الرقم بعد التأكد من عدم التكرار
        end
      end
      phoneCursor.close()
      contactInfo = contactInfo .. "\n" .. table.concat(phoneNumbers, "\n")
     else
      contactInfo = contactInfo .. "\nNo phone number found for this contact."
    end
    service.copy(contactInfo)
    Toast.makeText(activity, "Contact info copied to clipboard.", Toast.LENGTH_SHORT).show()
    service.click({{"%返回"}})
   else
    Toast.makeText(activity, "Contact not found.", Toast.LENGTH_SHORT).show()
  end
end



function openContactEditScreenFromText(activity, node)
  local text = service.getText(node)
  local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول
  contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
  if not contactName then
    Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
    return
  end
  local contentResolver = activity.getContentResolver()
  local cursor = contentResolver.query(
  ContactsContract.Contacts.CONTENT_URI,
  { ContactsContract.Contacts._ID },
  ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
  { contactName },
  nil
  )
  if cursor ~= nil and cursor.moveToFirst() then
    local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
    cursor.close()
    local editIntent = Intent(Intent.ACTION_EDIT)
    editIntent.setData(Uri.withAppendedPath(ContactsContract.Contacts.CONTENT_URI, contactId))
    editIntent.putExtra("finishActivityOnSaveCompleted", true)
    activity.startActivity(editIntent)
    service.click({{"%返回"}})
   else

    Toast.makeText(activity, "Contact not found", Toast.LENGTH_SHORT).show()
  end
end











function myBalance()
  function getPhoneNumberByName(name)
    local uri = ContactsContract.CommonDataKinds.Phone.CONTENT_URI
    local projection = {ContactsContract.CommonDataKinds.Phone.NUMBER}
    local selection = ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME .. " = ?"
    local selectionArgs = {name}
    local cursor = this.getContentResolver().query(uri, projection, selection, selectionArgs, nil)
    if cursor ~= nil and cursor.moveToFirst() then
      local numberIndex = cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER)
      local phoneNumber = cursor.getString(numberIndex)
      cursor.close()
      return phoneNumber
     else
      if cursor ~= nil then
        cursor.close()
      end
      return nil -- في حالة عدم العثور على الرقم
    end
  end
  local name = "رصيدي"
  local number = getPhoneNumberByName(name)
  if number ~= nil then
    local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
    this.startActivity(i)
   else
    print("لم يتم العثور على هذا الاسم.\n يجب إنشاء جهة اتصال جديدة باسم رصيدي وإدخال الرقم الذي من خلاله يتم معرفة الرصيد الخاص بك. ")
  end
end


function addNiewContact()
  intent = Intent(Intent.ACTION_INSERT)
  intent.setType(ContactsContract.Contacts.CONTENT_TYPE)
  this.startActivity(intent)
end
function showNumber()
  local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode("#31#"))) this.startActivity(i)
end
function hideNumber()
  local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode("*31#"))) this.startActivity(i)
end
function enableCallWaiting()
  service.click({{"#android:id/button1>10"}})
  task(500, function()
    local toneGenerator = ToneGenerator(AudioManager.STREAM_ACCESSIBILITY, 100)
    local function delay(ms)
      local endTime = os.clock() + (ms / 1000)
      while os.clock() < endTime do end
    end
    toneGenerator.startTone(ToneGenerator.TONE_SUP_RADIO_NOTAVAIL, 200)
    delay(600)
    toneGenerator.startTone(ToneGenerator.TONE_SUP_RADIO_NOTAVAIL, 200)
  end)
  local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode("*43#"))) this.startActivity(i)
end
function disableCallWaiting()
  service.click({{"#android:id/button1>10"}})
  task(500, function()
    local toneGenerator = ToneGenerator(AudioManager.STREAM_ACCESSIBILITY, 100)
    local toneGenerator = ToneGenerator(AudioManager.STREAM_ACCESSIBILITY, 100)
    local function delay(ms)
      local endTime = os.clock() + (ms / 1000)
      while os.clock() < endTime do end
    end
    toneGenerator.startTone(ToneGenerator.TONE_SUP_RINGTONE, 500)
    delay(900)
    toneGenerator.startTone(ToneGenerator.TONE_SUP_RINGTONE, 500)
  end)
  local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode("#43#"))) this.startActivity(i)
end


function useSpeakCallPhone()
  local prefs = PreferenceManager.getDefaultSharedPreferences(service)
  function setServiceEnabled(serviceName, activateService)
    local edit = prefs.edit()
    edit.putBoolean(serviceName, activateService)
    edit.commit()
  end
  local currentStatus = prefs.getBoolean("use_speak_call_phone", false)
  if currentStatus then
    setServiceEnabled("use_speak_call_phone", false)
    this.postSpeak(200, "عدم نُطق اسم المتصل ")
   else
    this.postSpeak(200, "نُطق اسم المتصل ")
    setServiceEnabled("use_speak_call_phone", true)
  end
end
function useSpeakCallTime()
  local prefs = PreferenceManager.getDefaultSharedPreferences(service)
  function setServiceEnabled(serviceName, activateService)
    local edit = prefs.edit()
    edit.putBoolean(serviceName, activateService)
    edit.commit()
  end
  local currentStatus = prefs.getBoolean("use_speak_call_time", false)
  if currentStatus then
    setServiceEnabled("use_speak_call_time", false)
    this.postSpeak(200, "عدم نُطق مدة المكالمة  ")
   else
    this.postSpeak(200, "نُطق مدة المكالمة  ")
    setServiceEnabled("use_speak_call_time", true)
  end
end



function phoneMode()
  context = activity or service
  audioManager = context.getSystemService(Context.AUDIO_SERVICE)
  notificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE)
  function getPhoneMode()
    local mode = audioManager.getRingerMode()
    if mode == AudioManager.RINGER_MODE_SILENT then
      return "Silent"
     elseif mode == AudioManager.RINGER_MODE_VIBRATE then
      return "Vibrate"
     elseif mode == AudioManager.RINGER_MODE_NORMAL then
      return "Ringer"
     else
      return "Unknown"
    end
  end
  function setVolume(streamType, progress)
    audioManager.setStreamVolume(streamType, progress, 0)
  end
  function setSoundMode(mode)
    local currentMode = audioManager.getRingerMode()
    if mode == "vibrate" and currentMode ~= AudioManager.RINGER_MODE_VIBRATE then
      audioManager.setRingerMode(AudioManager.RINGER_MODE_VIBRATE)
     elseif mode == "ring" and currentMode ~= AudioManager.RINGER_MODE_NORMAL then
      audioManager.setRingerMode(AudioManager.RINGER_MODE_NORMAL)
     elseif currentMode == AudioManager.RINGER_MODE_SILENT then
      print("The device is currently in silent mode.")
    end
  end
  function initializeSeekBar(seekBar, streamType)
    local maxVolume = audioManager.getStreamMaxVolume(streamType)
    local currentVolume = audioManager.getStreamVolume(streamType)
    seekBar.setMax(maxVolume)
    seekBar.setProgress(currentVolume)
    seekBar.setOnSeekBarChangeListener(luajava.createProxy("android.widget.SeekBar$OnSeekBarChangeListener", {
      onProgressChanged = function(seekBar, progress, fromUser)
        if fromUser then
          setVolume(streamType, progress)
        end
      end,
      onStartTrackingTouch = function(seekBar) end,
      onStopTrackingTouch = function(seekBar) end,
    }))
  end
  local currentMode = audioManager.getRingerMode()
  if currentMode == AudioManager.RINGER_MODE_NORMAL then
    setSoundMode("vibrate")
    service.postSpeak(500, "Phone mode: Vibrate")
   elseif currentMode == AudioManager.RINGER_MODE_VIBRATE then
    audioManager.setRingerMode(AudioManager.RINGER_MODE_NORMAL)
    service.playSoundTick()
    enableVibration()
    service.postSpeak(500, "Phone mode: Ringer")
   elseif currentMode == AudioManager.RINGER_MODE_SILENT then
    setSoundMode("ring")
    service.playSoundTick()
    enableVibration()
    service.postSpeak(500, "Phone mode: Ringer")
  end
  function enableVibration()
    if Build.VERSION.SDK_INT >= Build.VERSION_CODES.M then
      notificationManager.setInterruptionFilter(NotificationManager.INTERRUPTION_FILTER_ALL)
    end
  end
end
