function searchMapsEn()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition()
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "en-US")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "arabic" then
            this.postSpeak(200, "إِبْحَثْ في خرائط Google")
            task(1800, function() searchMapsAr() end)
          elseif text:lower() == "french" then
            this.postSpeak(200, "Recherche sur Google Maps")
            task(1800, function() searchMapsFr() end)
          elseif text:lower() == "english" then
            this.postSpeak(200, "Search on Google Maps")
            task(1800, function() searchMapsEn() end)
          else
            this.postSpeak(200, "Searching for " .. text)
            task(3500, function()
              local searchUrl = "https://www.google.com/maps/search/" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function() this.postSpeak(200, "No text detected.") end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition()
end

function searchMapsFr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition()
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "fr-FR")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "arabe" then
            this.postSpeak(200, "إِبْحَثْ في خرائط Google")
            task(1800, function() searchMapsAr() end)
          elseif text:lower() == "français" then
            this.postSpeak(200, "Recherche sur Google Maps")
            task(1800, function() searchMapsFr() end)
          elseif text:lower() == "anglais" then
            this.postSpeak(200, "Search on Google Maps")
            task(1800, function() searchMapsEn() end)
          else
            this.postSpeak(200, "Recherche en cours pour " .. text)
            task(3500, function()
              local searchUrl = "https://www.google.com/maps/search/" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function() this.postSpeak(200, "Aucun texte n'a été détecté.") end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition()
end

function searchMapsAr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition()
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ar-SA")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "عربي" then
            this.postSpeak(200, "إِبْحَثْ في خرائط Google")
            task(1800, function() searchMapsAr() end)
          elseif text:lower() == "فرنسي" then
            this.postSpeak(200, "Recherche sur Google Maps")
            task(1800, function() searchMapsFr() end)
          elseif text:lower() == "انجليزي" then
            this.postSpeak(200, "Search on Google Maps")
            task(1800, function() searchMapsEn() end)
          elseif text:lower() == "توقف" then
            this.postSpeak(200, "تم إيقافُ البحث.")
            return
          else
            this.postSpeak(200, "جاري البحث عن " .. text)
            task(3500, function()
              local searchUrl = "https://www.google.com/maps/search/" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function() this.postSpeak(200, "لم يتم التقاط أي نص.") end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition()
end






function searchGoogleEn()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "en-US")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "arabic" then
            this.postSpeak(200, "إِبْحَثْ في Google")
            require "import"
            task(1800, function() searchGoogleAr() end)
            return
          elseif text:lower() == "french" then
            this.postSpeak(200, "Recherche sur Google")
            require "import"
            task(1800, function() searchGoogleFr() end)
            return
          elseif text:lower() == "english" then
            this.postSpeak(200, "Search on Google")
            require "import"
            task(1800, function() searchGoogleEn() end)
            return
          else
            this.postSpeak(200, "Searching for " .. text)
            require "import"
            task(3500, function()
              local searchUrl = "https://www.google.com/search?q=" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "No text detected.")
      end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end

function searchGoogleFr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "fr-Fr")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "arabe" then
            this.postSpeak(200, "إِبْحَثْ في Google")
            require "import"
            task(1800, function() searchGoogleAr() end)
            return
          elseif text:lower() == "français" then
            this.postSpeak(200, "Recherche sur Google")
            require "import"
            task(1800, function() searchGoogleFr() end)
            return
          elseif text:lower() == "anglais" then
            this.postSpeak(200, "Search on Google")
            require "import"
            task(1800, function() searchGoogleEn() end)
            return
          else
            this.postSpeak(200, "Recherche en cours pour " .. text)
            require "import"
            task(3500, function()
              local searchUrl = "https://www.google.com/search?q=" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "Aucun texte n'a été détecté.")
      end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end

function searchGoogleAr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ar-SA")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "عربي" then
            this.postSpeak(200, "إِبْحَثْ في Google")
            require "import"
            task(1800, function() searchGoogleAr() end)
            return
          elseif text:lower() == "فرنسي" then
            this.postSpeak(200, "Recherche sur Google")
            require "import"
            task(1800, function() searchGoogleFr() end)
            return
          elseif text:lower() == "انجليزي" then
            this.postSpeak(200, "Search on Google")
            require "import"
            task(1800, function() searchGoogleEn() end)
            return
          elseif text:lower() == "توقف" then
            this.postSpeak(200, "تم إيقافُ البحث.")
            return
          else
            this.postSpeak(200, "جاري البحث عن " .. text)
            require "import"
            task(3500, function()
              local searchUrl = "https://www.google.com/search?q=" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end





function searchGooglePlayEn()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "en-US")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "arabic" then
            this.postSpeak(200, "إِبْحَثْ في Play Store")
            require "import"
            task(1800, function() searchGooglePlayAr() end)
            return
           elseif text:lower() == "french" then
            this.postSpeak(200, "Recherche sur Google Play")
            require "import"
            task(1800, function() searchGooglePlayFr() end)
            return
           elseif text:lower() == "english" then
            this.postSpeak(200, "Search on Google Play")
            require "import"
            task(1800, function() searchGooglePlayEn() end)
            return
           else
            this.postSpeak(200, "Searching for " .. text)
            require "import"
            task(3500, function()
              local searchUrl = "https://play.google.com/store/search?q=" .. Uri.encode(text) .. "&c=apps"
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end

function searchGooglePlayFr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "fr-Fr")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "arabe" then
            this.postSpeak(200, "إِبْحَثْ في Play Store")
            require "import"
            task(1800, function() searchGooglePlayAr() end)
            return
           elseif text:lower() == "français" then
            this.postSpeak(200, "Recherche sur Google Play")
            require "import"
            task(1800, function() searchGooglePlayFr() end)
            return
           elseif text:lower() == "anglais" then
            this.postSpeak(200, "Search on Google Play")
            require "import"
            task(1800, function() searchGooglePlayEn() end)
            return
           else
            this.postSpeak(200, "Recherche en cours pour " .. text)
            require "import"
            task(3500, function()
              local searchUrl = "https://play.google.com/store/search?q=" .. Uri.encode(text) .. "&c=apps"
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "Aucun texte n'a été détecté.")
      end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end

function searchGooglePlayAr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ar-SA")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "عربي" then
            this.postSpeak(200, "إِبْحَثْ في Play Store")
            require "import"
            task(1800, function() searchGooglePlayAr() end)
            return
           elseif text:lower() == "فرنسي" then
            this.postSpeak(200, "Recherche sur Google Play")
            require "import"
            task(1800, function() searchGooglePlayFr() end)
            return
           elseif text:lower() == "انجليزي" then
            this.postSpeak(200, "Search on Google Play")
            require "import"
            task(1800, function() searchGooglePlayEn() end)
            return
          elseif text:lower() == "توقف" then
            this.postSpeak(200, "تم إيقافُ البحث.")
            return
           else

            this.postSpeak(200, "جاري البحث عن " .. text)
            require "import"
            task(3500, function()
              local searchUrl = "https://play.google.com/store/search?q=" .. Uri.encode(text) .. "&c=apps"
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end





function searchFacebookEn()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "en-US")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "arabic" then
            this.postSpeak(200, "إِبْحَثْ في Facebook")
            require "import"
            task(1800, function() searchFacebookAr() end)
            return
           elseif text:lower() == "french" then
            this.postSpeak(200, "Recherche sur Facebook")
            require "import"
            task(1800, function() searchFacebookFr() end)
            return
           elseif text:lower() == "english" then
            this.postSpeak(200, "Search on Facebook")
            require "import"
            task(1800, function() searchFacebookEn() end)
            return
           else
            this.postSpeak(200,"Searching for " .. text)
            require "import"
            task(3500, function()

              local searchUrl = "https://www.facebook.com/search/top/?q=" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end




function searchFacebookFr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "fr-Fr")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "Facebook arabe" then
            this.postSpeak(200, "إِبْحَثْ في Facebook")
            require "import"
            task(1800, function() searchFacebookAr() end)
            return
           elseif text:lower() == "Facebook français" then
            this.postSpeak(200, "Recherche sur Facebook")
            require "import"
            task(1800, function() searchFacebookFr() end)
            return
           elseif text:lower() == "Facebook anglais" then
            this.postSpeak(200, "Search on Facebook")
            require "import"
            task(1800, function() searchFacebookEn() end)
            return
           else
            this.postSpeak(200,"Recherche en cours pour " .. text)
            require "import"
            task(3500, function()

              local searchUrl = "https://www.facebook.com/search/top/?q=" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end




function searchFacebookAr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local Uri = luajava.bindClass("android.net.Uri")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ar-SA")
    handler.postDelayed(function() vibrator.vibrate(50) end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)
        if text and text ~= "" then
          if text:lower() == "فيسبوك عربي" then
            this.postSpeak(200, "إِبْحَثْ في Facebook")
            require "import"
            task(1800, function() searchFacebookAr() end)
            return
           elseif text:lower() == "فيسبوك فرنسي" then
            this.postSpeak(200, "Recherche sur Facebook")
            require "import"
            task(1800, function() searchFacebookFr() end)
            return
           elseif text:lower() == "فيسبوك انجليزي" then
            this.postSpeak(200, "Search on Facebook")
            require "import"
            task(1800, function() searchFacebookEn() end)
            return
          elseif text:lower() == "توقف" then
            this.postSpeak(200, "تم إيقافُ البحث.")
            return
           else
            this.postSpeak(200,"جاري البحث عن "..text)
            require "import"
            task(3500, function()

              local searchUrl = "https://www.facebook.com/search/top/?q=" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end


function searchYouTubeAr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper") -- تم إضافة هذا السطر لاستدعاء Looper
  local Handler = luajava.bindClass("android.os.Handler") -- تأكد من استدعاء Handler بشكل صحيح أيضاً
  local Uri = luajava.bindClass("android.net.Uri") -- تم إضافة Uri
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ar-SA")

    handler.postDelayed(function()
      vibrator.vibrate(50)
    end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)

        if text and text ~= "" then
          if text:lower() == "عربي" then
            this.postSpeak(200, "إِبْحَثْ في YouTube")
            require "import"
            task(1800, function()
              searchYouTubeAr()
            end)
            return
           elseif text:lower() == "فرنسي" then
            this.postSpeak(200, "Recherche sur YouTube")
            require "import"
            task(1800, function()
              searchYouTubeFr()
            end)
            return
           elseif text:lower() == "انجليزي" then
            this.postSpeak(200, "Search on YouTube")
            require "import"
            task(1800, function()
              searchYouTubeEn()
            end)
            return
          elseif text:lower() == "توقف" then
            this.postSpeak(200, "تم إيقافُ البحث.")
            return
           else
            this.postSpeak(200,"جاري البحث عن "..text)
            require "import"
            task(3500, function()
              local searchUrl = "https://www.youtube.com/results?search_query=" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end,
      -- يمكنك إضافة المزيد من الحالات الخاصة بـ onReadyForSpeech وغيرها إذا أردت
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end




function searchYouTubeFr()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper") -- تم إضافة هذا السطر لاستدعاء Looper
  local Handler = luajava.bindClass("android.os.Handler") -- تأكد من استدعاء Handler بشكل صحيح أيضاً
  local Uri = luajava.bindClass("android.net.Uri") -- تم إضافة Uri
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "fr-Fr")

    handler.postDelayed(function()
      vibrator.vibrate(50)
    end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)

        if text and text ~= "" then
          if text:lower() == "arabe" then
            this.postSpeak(200, "إِبْحَثْ في YouTube")
            require "import"
            task(1800, function()
              searchYouTubeAr()
            end)
            return
           elseif text:lower() == "français" then
            this.postSpeak(200, "Recherche sur YouTube")
            require "import"
            task(1800, function()
              searchYouTubeFr()
            end)
            return
           elseif text:lower() == "anglais" then
            this.postSpeak(200, "Search on YouTube")
            require "import"
            task(1800, function()
              searchYouTubeEn()
            end)
            return
           else
            this.postSpeak(200,"Recherche en cours pour " .. text)
            require "import"
            task(3500, function()
              local searchUrl = "https://www.youtube.com/results?search_query=" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end,
      -- يمكنك إضافة المزيد من الحالات الخاصة بـ onReadyForSpeech وغيرها إذا أردت
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end




function searchYouTubeEn()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper") -- تم إضافة هذا السطر لاستدعاء Looper
  local Handler = luajava.bindClass("android.os.Handler") -- تأكد من استدعاء Handler بشكل صحيح أيضاً
  local Uri = luajava.bindClass("android.net.Uri") -- تم إضافة Uri
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "en-US")

    handler.postDelayed(function()
      vibrator.vibrate(50)
    end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)

        if text and text ~= "" then
          if text:lower() == "arabic" then
            this.postSpeak(200, "إِبْحَثْ في YouTube")
            require "import"
            task(1800, function()
              searchYouTubeAr()
            end)
            return
           elseif text:lower() == "french" then
            this.postSpeak(200, "Recherche sur YouTube")
            require "import"
            task(1800, function()
              searchYouTubeFr()
            end)
            return
           elseif text:lower() == "english" then
            this.postSpeak(200, "Search on YouTube")
            require "import"
            task(1800, function()
              searchYouTubeEn()
            end)
            return
           else
            this.postSpeak(200,"Searching for " .. text)
            require "import"
            task(3500, function()
              local searchUrl = "https://www.youtube.com/results?search_query=" .. Uri.encode(text)
              local intent = Intent(Intent.ACTION_VIEW, Uri.parse(searchUrl))
              activity.startActivity(intent)
            end)
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end,
      -- يمكنك إضافة المزيد من الحالات الخاصة بـ onReadyForSpeech وغيرها إذا أردت
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  startVoiceRecognition(activity)
end





function searchSpecificService()
  local luajava = require("luajava")
  local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  local Intent = luajava.bindClass("android.content.Intent")
  local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  local Looper = luajava.bindClass("android.os.Looper")
  local Handler = luajava.bindClass("android.os.Handler")
  local activity = this
  local handler = Handler(Looper.getMainLooper())
  local vibrator = activity.getSystemService("vibrator")
  local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)

  local function startVoiceRecognition(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ar")

    handler.postDelayed(function()
      vibrator.vibrate(50)
    end, 350)

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
      onResults = function(params)
        local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        local text = results.get(0)

        if text and text ~= "" then
          if text:lower() == "يوتيوب عربي" then
            this.postSpeak(200, "إِبْحَثْ في YouTube")
            require "import"
            task(1800, function()
              searchYouTubeAr()
            end)
            return
           elseif text:lower() == "يوتيوب فرنسي" then
            this.postSpeak(200, "Recherche sur YouTube")
            require "import"
            task(1800, function()
              searchYouTubeFr()
            end)
            return
           elseif text:lower() == "يوتيوب انجليزي" then
            this.postSpeak(200, "Search on YouTube")
            require "import"
            task(1800, function()
              searchYouTubeEn()
            end)
            return
           elseif text:lower() == "فيسبوك عربي" then
            this.postSpeak(200, "إِبْحَثْ في Facebook")
            require "import"
            task(1800, function() searchFacebookAr() end)
            return
           elseif text:lower() == "فيسبوك فرنسي" then
            this.postSpeak(200, "Recherche sur Facebook")
            require "import"
            task(1800, function() searchFacebookFr() end)
            return
           elseif text:lower() == "فيسبوك انجليزي" then
            this.postSpeak(200, "Search on Facebook")
            require "import"
            task(1800, function() searchFacebookEn() end)
            return

           elseif text:lower() == "تطبيق عربي" then
            this.postSpeak(200, "إِبْحَثْ في Play Store")
            require "import"
            task(1800, function() searchGooglePlayAr() end)
            return
           elseif text:lower() == "تطبيق فرنسي" then
            this.postSpeak(200, "Recherche sur Google Play")
            require "import"
            task(1800, function() searchGooglePlayFr() end)
            return
           elseif text:lower() == "تطبيق انجليزي" then
            this.postSpeak(200, "Search on Google Play")
            require "import"
            task(1800, function() searchGooglePlayEn() end)
            return

          elseif text:lower() == "جوجل عربي" then
            this.postSpeak(200, "إِبْحَثْ في Google")
            require "import"
            task(1800, function() searchGoogleAr() end)
            return
          elseif text:lower() == "جوجل فرنسي" then
            this.postSpeak(200, "Recherche sur Google")
            require "import"
            task(1800, function() searchGoogleFr() end)
            return
          elseif text:lower() == "جوجل انجليزي" then
            this.postSpeak(200, "Search on Google")
            require "import"
            task(1800, function() searchGoogleEn() end)
            return

elseif text:lower() == "خرائط عربي" then
  this.postSpeak(200, "إِبْحَثْ في خرائط Google")
  require "import"
  task(1800, function() searchMapsAr() end)
  return
elseif text:lower() == "خرائط فرنسي" then
  this.postSpeak(200, "Recherche sur Google Maps")
  require "import"
  task(1800, function() searchMapsFr() end)
  return
elseif text:lower() == "خرائط انجليزي" then
  this.postSpeak(200, "Search on Google Maps")
  require "import"
  task(1800, function() searchMapsEn() end)
  return

          elseif text:lower() == "توقف" then
            this.postSpeak(200, "تم إيقافُ البحث.")
            return
           else
            this.postSpeak(200, "لا توجد خدمة متطابقة.")
          end
        end
      end,
      onError = function(error)
        this.postSpeak(200, "لم يتم التقاط أي نص.")
      end,
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
  end

  service.speak("إخترِ التطبيقْ, ولغةَ البحثْ")
  require "import"
  task(2200, function()
    startVoiceRecognition(activity)
  end)
end

searchSpecificService()