
function checkContactsAccess()
  local activity = this
  local resolver = activity.getContentResolver()
  local hasPermission = activity.checkSelfPermission("android.permission.READ_CONTACTS") == activity.PackageManager.PERMISSION_GRANTED
  if not hasPermission then
    activity.requestPermissions({"android.permission.READ_CONTACTS"}, 1)
    return "Permission required to access contacts."
  end
  local cursor = resolver.query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, {ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME, ContactsContract.CommonDataKinds.Phone.NUMBER}, nil, nil, nil)
  local contactsList = {}
  if cursor then
    while cursor.moveToNext() do
      local name = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME))
      local number = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
      contactsList[normalizeNumber(number)] = name -- تخزين الرقم بدون مسافات
    end
    cursor.close()
   else
    return "Failed to retrieve contacts."
  end
  return contactsList
end

function getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri, {CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE, CallLog.Calls.DURATION}, nil, nil, CallLog.Calls.DATE .. " DESC")
  local resultsList = {}
  if cursor then
    while cursor.moveToNext() do
      local number = normalizeNumber(cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER)))
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
      local duration = tonumber(cursor.getString(cursor.getColumnIndex(CallLog.Calls.DURATION))) -- الحصول على مدة المكالمة وتحويلها إلى رقم

      local typeString = ""
      if type == tostring(CallLog.Calls.INCOMING_TYPE) then
        typeString = "incoming"
       elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
        typeString = "outgoing"
       elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
        typeString = "missed"
       elseif type == tostring(CallLog.Calls.REJECTED_TYPE) then
        typeString = "Rejected"
      end

      local dateNumber = tonumber(date)
      local seconds = math.floor(dateNumber / 1000)
      local formattedDate = os.date("%d-%m-%Y %H:%M", seconds)
      local contactName = contactsList[number] or "unknown"

      local hours = math.floor(duration / 3600)
      local minutes = math.floor((duration % 3600) / 60)
      local secs = duration % 60
      local formattedDuration = string.format("%02d:%02d:%02d", hours, minutes, secs)

      table.insert(resultsList, {
        name = contactName,
        type = typeString,
        date = formattedDate,
        number = number,
        duration = formattedDuration
      })
    end
    cursor.close()
   else
    print("لا يوجد سجل مكالمات.")
  end
  return resultsList
end

function formatResults(results)
  local formatted = {}
  if results then
    for _, result in ipairs(results) do
      local entry = result.name .. ", \n" .. result.type .. ", \n" .. result.number .. ". \n" .. result.date .. ". \n" .. result.duration
      table.insert(formatted, entry)
    end
  end
  return formatted
end


function normalizeNumber(number)
  return number:gsub("%s+", "") -- إزالة المسافات
end







function getLastCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToFirst() then
    local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
    local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
    local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
    local typeString = ""
    if type == tostring(CallLog.Calls.INCOMING_TYPE) then
      typeString = "incoming"
     elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
      typeString = "outgoing"
     elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
      typeString = "missed"
    end
    local dateNumber = tonumber(date)
    local seconds = math.floor(dateNumber / 1000)
    local contactName = contactsList[number] or "unknown"
    this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
    cursor.close()
   else
    this.postSpeak(200, "No call history available.")
  end
end



function getSecondLastCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToPosition(1) then
    local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
    local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
    local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
    local typeString = ""
    if type == tostring(CallLog.Calls.INCOMING_TYPE) then
      typeString = "incoming"
     elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
      typeString = "outgoing"
     elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
      typeString = "missed"
    end
    local dateNumber = tonumber(date)
    local seconds = math.floor(dateNumber / 1000)
    local contactName = contactsList[number] or "unknown"
    this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
    cursor.close()
   else
    this.postSpeak(200, "No call history available.")
  end
end



function getLastMissedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor then
    while cursor.moveToNext() do
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      if type == tostring(CallLog.Calls.MISSED_TYPE) then
        local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
        local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
        local typeString = "missed"
        local dateNumber = tonumber(date)
        local seconds = math.floor(dateNumber / 1000)
        local contactName = contactsList[number] or "unknown"
        this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
        break
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No missed calls.")
  end
end



function getSecondLastMissedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor then
    local missedCallCount = 0
    while cursor.moveToNext() do
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      if type == tostring(CallLog.Calls.MISSED_TYPE) then
        missedCallCount = missedCallCount + 1
        if missedCallCount == 2 then
          local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
          local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
          local typeString = "missed"
          local dateNumber = tonumber(date)
          local seconds = math.floor(dateNumber / 1000)
          local contactName = contactsList[number] or "unknown"
          this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
          break
        end
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No missed calls.")
  end
end



function getLastRejectedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor then
    while cursor.moveToNext() do
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      if type == tostring(CallLog.Calls.REJECTED_TYPE) then
        local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
        local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
        local typeString = "Rejected"
        local dateNumber = tonumber(date)
        local seconds = math.floor(dateNumber / 1000)
        local contactName = contactsList[number] or "unknown"
        this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
        break
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No Rejected calls.")
  end
end



function getSecondLastRejectedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  local foundFirstRejected = false
  if cursor then
    while cursor.moveToNext() do
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      if type == tostring(CallLog.Calls.REJECTED_TYPE) then
        if foundFirstRejected then
          local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
          local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
          local typeString = "Rejected"
          local dateNumber = tonumber(date)
          local seconds = math.floor(dateNumber / 1000)
          local contactName = contactsList[number] or "unknown"
          this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
          break
         else
          foundFirstRejected = true
        end
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No Rejected calls.")
  end
end



function getLastIncomingCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  CallLog.Calls.TYPE .. " = ?",
  { tostring(CallLog.Calls.INCOMING_TYPE) },
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToFirst() then
    local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
    local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
    local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
    local typeString = ""
    if type == tostring(CallLog.Calls.INCOMING_TYPE) then
      typeString = "incoming"
     elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
      typeString = "outgoing"
     elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
      typeString = "missed"
    end
    local dateNumber = tonumber(date)
    local seconds = math.floor(dateNumber / 1000)
    local contactName = contactsList[number] or "unknown"
    this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
    cursor.close()
   else
    this.postSpeak(200, "No incoming calls.")
  end
end



function getSecondLastIncomingCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  CallLog.Calls.TYPE .. " = ?",
  { tostring(CallLog.Calls.INCOMING_TYPE) },
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToNext() then
    cursor.moveToFirst()
    if cursor.moveToNext() then
      local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
      local typeString = ""
      if type == tostring(CallLog.Calls.INCOMING_TYPE) then
        typeString = "incoming"
       elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
        typeString = "outgoing"
       elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
        typeString = "missed"
      end
      local dateNumber = tonumber(date)
      local seconds = math.floor(dateNumber / 1000)
      local contactName = contactsList[number] or "unknown"
      this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
     else
      this.postSpeak(200, "No second-to-last incoming call.")
    end
    cursor.close()
   else
    this.postSpeak(200, "No incoming calls.")
  end
end



function getLastOutgoingCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  CallLog.Calls.TYPE .. " = ?",
  { tostring(CallLog.Calls.OUTGOING_TYPE) },
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToFirst() then
    local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
    local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
    local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
    local typeString = ""
    if type == tostring(CallLog.Calls.INCOMING_TYPE) then
      typeString = "incoming"
     elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
      typeString = "outgoing"
     elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
      typeString = "missed"
    end
    local dateNumber = tonumber(date)
    local seconds = math.floor(dateNumber / 1000)
    local contactName = contactsList[number] or "unknown"
    this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
    cursor.close()
   else
    this.postSpeak(200, "No outgoing calls.")
  end
end



function getSecondLastOutgoingCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  CallLog.Calls.TYPE .. " = ?",
  { tostring(CallLog.Calls.OUTGOING_TYPE) },
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToNext() then
    cursor.moveToFirst()
    if cursor.moveToNext() then
      local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
      local typeString = ""
      if type == tostring(CallLog.Calls.INCOMING_TYPE) then
        typeString = "incoming"
       elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
        typeString = "outgoing"
       elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
        typeString = "missed"
      end
      local dateNumber = tonumber(date)
      local seconds = math.floor(dateNumber / 1000)
      local contactName = contactsList[number] or "unknown"
      this.postSpeak(200, contactName .. ", " .. number .. ". " .. typeString .. ", " .. os.date("%d-%m-%Y %H:%M:%S", seconds))
     else
      this.postSpeak(200, "No second-to-last outgoing call.")
    end
    cursor.close()
   else
    this.postSpeak(200, "No outgoing calls.")
  end
end



function getLastUnnamedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor then
    while cursor.moveToNext() do
      local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
      local contactName = contactsList[number]
      if not contactName then
        local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
        local dateNumber = tonumber(date)
        local seconds = math.floor(dateNumber / 1000)
        local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
        local typeString = (type == tostring(CallLog.Calls.INCOMING_TYPE)) and "incoming"
        or (type == tostring(CallLog.Calls.OUTGOING_TYPE)) and "outgoing"
        or (type == tostring(CallLog.Calls.MISSED_TYPE)) and "missed"
        or (type == tostring(CallLog.Calls.REJECTED_TYPE)) and "Rejected"
        or "unknown"
        this.postSpeak(200, "unknown: " .. number .. ", : " .. typeString .. ", " .. os.date("%Y-%m-%d %H:%M:%S", seconds))
        break
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No unknown calls.")
  end
end



function getSecondLastUnnamedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  local foundFirstUnnamed = false
  if cursor then
    while cursor.moveToNext() do
      local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
      local contactName = contactsList[number]
      if not contactName then
        if foundFirstUnnamed then
          local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
          local dateNumber = tonumber(date)
          local seconds = math.floor(dateNumber / 1000)
          local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
          local typeString = (type == tostring(CallLog.Calls.INCOMING_TYPE)) and "incoming"
          or (type == tostring(CallLog.Calls.OUTGOING_TYPE)) and "outgoing"
          or (type == tostring(CallLog.Calls.MISSED_TYPE)) and "missed"
          or (type == tostring(CallLog.Calls.REJECTED_TYPE)) and "Rejected"
          or "unknown"
          this.postSpeak(200, "unknown: " .. number .. ", : " .. typeString .. ", " .. os.date("%Y-%m-%d %H:%M:%S", seconds))
          break
         else
          foundFirstUnnamed = true
        end
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No unknown calls.")
  end
end






function callLastCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToFirst() then
    local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
    local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
    local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
    local typeString = ""
    if type == tostring(CallLog.Calls.INCOMING_TYPE) then
      typeString = "incoming"
     elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
      typeString = "outgoing"
     elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
      typeString = "missed"
    end
    local dateNumber = tonumber(date)
    local seconds = math.floor(dateNumber / 1000)
    local contactName = contactsList[number] or "unknown"
    local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
    this.startActivity(i)
    cursor.close()
   else
    this.postSpeak(200, "No call history available.")
  end
end




function callSecondLastCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToPosition(1) then
    local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
    local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
    local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
    local typeString = ""
    if type == tostring(CallLog.Calls.INCOMING_TYPE) then
      typeString = "incoming"
     elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
      typeString = "outgoing"
     elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
      typeString = "missed"
    end
    local dateNumber = tonumber(date)
    local seconds = math.floor(dateNumber / 1000)
    local contactName = contactsList[number] or "unknown"
    local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
    this.startActivity(i)
    cursor.close()
   else
    this.postSpeak(200, "No call history available.")
  end
end



function callLastMissedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor then
    while cursor.moveToNext() do
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      if type == tostring(CallLog.Calls.MISSED_TYPE) then
        local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
        local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
        local typeString = "missed"
        local dateNumber = tonumber(date)
        local seconds = math.floor(dateNumber / 1000)
        local contactName = contactsList[number] or "unknown"
        local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
        this.startActivity(i)
        break
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No missed calls.")
  end
end



function callSecondLastMissedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor then
    local missedCallCount = 0
    while cursor.moveToNext() do
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      if type == tostring(CallLog.Calls.MISSED_TYPE) then
        missedCallCount = missedCallCount + 1
        if missedCallCount == 2 then
          local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
          local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
          local typeString = "missed"
          local dateNumber = tonumber(date)
          local seconds = math.floor(dateNumber / 1000)
          local contactName = contactsList[number] or "unknown"
          local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
          this.startActivity(i)
          break
        end
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No missed calls.")
  end
end



function callLastRejectedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor then
    while cursor.moveToNext() do
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      if type == tostring(CallLog.Calls.REJECTED_TYPE) then
        local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
        local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
        local typeString = "Rejected"
        local dateNumber = tonumber(date)
        local seconds = math.floor(dateNumber / 1000)
        local contactName = contactsList[number] or "unknown"
        local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
        this.startActivity(i)
        break
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No Rejected calls.")
  end
end



function callSecondLastRejectedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  local foundFirstRejected = false
  if cursor then
    while cursor.moveToNext() do
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      if type == tostring(CallLog.Calls.REJECTED_TYPE) then
        if foundFirstRejected then
          local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
          local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
          local typeString = "Rejected"
          local dateNumber = tonumber(date)
          local seconds = math.floor(dateNumber / 1000)
          local contactName = contactsList[number] or "unknown"
          local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
          this.startActivity(i)
          break
         else
          foundFirstRejected = true
        end
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No Rejected calls.")
  end
end



function callLastIncomingCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  CallLog.Calls.TYPE .. " = ?",
  { tostring(CallLog.Calls.INCOMING_TYPE) },
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToFirst() then
    local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
    local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
    local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
    local typeString = ""
    if type == tostring(CallLog.Calls.INCOMING_TYPE) then
      typeString = "incoming"
     elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
      typeString = "outgoing"
     elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
      typeString = "missed"
    end
    local dateNumber = tonumber(date)
    local seconds = math.floor(dateNumber / 1000)
    local contactName = contactsList[number] or "unknown"
    local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
    this.startActivity(i)
    cursor.close()
   else
    this.postSpeak(200, "No incoming calls.")
  end
end



function callSecondLastIncomingCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  CallLog.Calls.TYPE .. " = ?",
  { tostring(CallLog.Calls.INCOMING_TYPE) },
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToNext() then
    cursor.moveToFirst()
    if cursor.moveToNext() then
      local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
      local typeString = ""
      if type == tostring(CallLog.Calls.INCOMING_TYPE) then
        typeString = "incoming"
       elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
        typeString = "outgoing"
       elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
        typeString = "missed"
      end
      local dateNumber = tonumber(date)
      local seconds = math.floor(dateNumber / 1000)
      local contactName = contactsList[number] or "unknown"
      local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
      this.startActivity(i)
     else
      this.postSpeak(200, "No second-to-last incoming call.")
    end
    cursor.close()
   else
    this.postSpeak(200, "No incoming calls.")
  end
end



function callLastOutgoingCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  CallLog.Calls.TYPE .. " = ?",
  { tostring(CallLog.Calls.OUTGOING_TYPE) },
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToFirst() then
    local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
    local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
    local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
    local typeString = ""
    if type == tostring(CallLog.Calls.INCOMING_TYPE) then
      typeString = "incoming"
     elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
      typeString = "outgoing"
     elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
      typeString = "missed"
    end
    local dateNumber = tonumber(date)
    local seconds = math.floor(dateNumber / 1000)
    local contactName = contactsList[number] or "unknown"
    local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
    this.startActivity(i)
    cursor.close()
   else
    this.postSpeak(200, "No outgoing calls.")
  end
end



function callSecondLastOutgoingCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  CallLog.Calls.TYPE .. " = ?",
  { tostring(CallLog.Calls.OUTGOING_TYPE) },
  CallLog.Calls.DATE .. " DESC")
  if cursor and cursor.moveToNext() then
    cursor.moveToFirst()
    if cursor.moveToNext() then
      local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
      local typeString = ""
      if type == tostring(CallLog.Calls.INCOMING_TYPE) then
        typeString = "incoming"
       elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
        typeString = "outgoing"
       elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
        typeString = "missed"
      end
      local dateNumber = tonumber(date)
      local seconds = math.floor(dateNumber / 1000)
      local contactName = contactsList[number] or "unknown"
      local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
      this.startActivity(i)
     else
      this.postSpeak(200, "No second-to-last outgoing call.")
    end
    cursor.close()
   else
    this.postSpeak(200, "No outgoing calls.")
  end
end



function callLastUnnamedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  if cursor then
    while cursor.moveToNext() do
      local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
      local contactName = contactsList[number]
      if not contactName then
        local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
        local dateNumber = tonumber(date)
        local seconds = math.floor(dateNumber / 1000)
        local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
        local typeString = (type == tostring(CallLog.Calls.INCOMING_TYPE)) and "incoming"
        or (type == tostring(CallLog.Calls.OUTGOING_TYPE)) and "outgoing"
        or (type == tostring(CallLog.Calls.MISSED_TYPE)) and "missed"
        or (type == tostring(CallLog.Calls.REJECTED_TYPE)) and "Rejected"
        or "unknown"
        local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
        this.startActivity(i)
        break
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No unknown calls.")
  end
end



function callSecondLastUnnamedCallLog(contactsList)
local contactsList = checkContactsAccess()
local resultsList = getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri,
  { CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE },
  nil,
  nil,
  CallLog.Calls.DATE .. " DESC")
  local foundFirstUnnamed = false
  if cursor then
    while cursor.moveToNext() do
      local number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER))
      local contactName = contactsList[number]
      if not contactName then
        if foundFirstUnnamed then
          local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
          local dateNumber = tonumber(date)
          local seconds = math.floor(dateNumber / 1000)
          local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
          local typeString = (type == tostring(CallLog.Calls.INCOMING_TYPE)) and "incoming"
          or (type == tostring(CallLog.Calls.OUTGOING_TYPE)) and "outgoing"
          or (type == tostring(CallLog.Calls.MISSED_TYPE)) and "missed"
          or (type == tostring(CallLog.Calls.REJECTED_TYPE)) and "Rejected"
          or "unknown"
          local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
          this.startActivity(i)
          break
         else
          foundFirstUnnamed = true
        end
      end
    end
    cursor.close()
   else
    this.postSpeak(200, "No unknown calls.")
  end
end