require "import"
import "android.widget.*"
import "android.view.Gravity"
require "quran"

dir = package.searchpath("main", package.path):match("(.*)main.lua")

function removeDiacritics(text)
  local diacritics = "[ًٌٍَُِّْـ]"
  return (text:gsub(diacritics, ""))
end

filteredItems = {}
filteredIndexes = {}
currentItems = {}
is_bahth_tam = false

verseToSurahMap = {}

function search(id, dataSource)
  id.addTextChangedListener({
    onTextChanged = function(s)
      local searchText = tostring(s):gsub("\n", "")
      filteredItems = {}
      filteredIndexes = {}
      if searchText ~= "" then
        is_bahth_tam = true
        for index, item in ipairs(dataSource) do
          local plainText = item
          if dataSource == allVerses then
            plainText = item:match("^(.-) %[%d+%] %- .+$") or item
          end
          if removeDiacritics(plainText):find(removeDiacritics(searchText), 1, true) then
            table.insert(filteredItems, item)
          end
        end
      else
        is_bahth_tam = false
        filteredItems = dataSource
      end

      adapter.clear()
      for _, item in ipairs(filteredItems) do
        adapter.add(item)
      end
      adapter.notifyDataSetChanged()

      if customTitle then
        if searchText ~= "" then
          customTitle.setText("نتائج البحث: " .. #filteredItems)
        else
          customTitle.setText("الباحث القرآني")
        end
      end

      currentItems = dataSource
    end
  })
end

main_layout = {
  LinearLayout;
  orientation = "vertical";
  layout_width = "match_parent";
  layout_height = "match_parent";

  { EditText,
    hint = "إبحث في القرآن",
    id = "search_quran_id",
    layout_width = "match_parent",
  },

  { EditText,
    hint = "إبحث في القائمة الحالية",
    id = "search_list_id",
    layout_width = "match_parent",
  },

  { Button,
    id = "delete_button",
    text = "حذف المعلومات أو الآيات",
    layout_width = "match_parent",
    layout_height = "wrap_content",
    textColor = 0xFFFFFFFF,
  },

  { TextView,
    id = "customTitle",
    layout_width = "match_parent",
    layout_height = "wrap_content",
    textSize = 20,
    gravity = "center",
    padding = "20",
    text = "الباحث القرآني",
  },

  { ListView,
    id = "list",
    layout_width = "match_parent",
    layout_height = 0,
    layout_weight = 1,
  },
}

dlg = LuaDialog()
dlg.view = loadlayout(main_layout)

-- تعيين الألوان بعد التحميل لتفادي الخطأ
delete_button.setBackgroundColor(0xFF8B0000) -- أحمر قاتم جداً
search_list_id.setBackgroundColor(0xFF004D00)
search_list_id.setTextColor(0xFFFFFFFF)
search_quran_id.setBackgroundColor(0xFF004D00)
search_quran_id.setTextColor(0xFFFFFFFF)
list.setBackgroundColor(0xFF000000)
customTitle.setTextColor(0xFFFFFFFF)
customTitle.setBackgroundColor(0xFF000000)

-- وظائف الزر الجديد
delete_button.onClick = function()
    dlg.dismiss()
    task(400, function()
dofile(dir.. "removeBracketInfoFromLines.lua")
    end)
end

delete_button.onLongClick = function()
    dlg.dismiss()
    task(800, function()
dofile(dir.. "deleteAllVerses.lua")
    end)
  return true
end

allVerses = {}
for _, surahName in ipairs(ordered_surahs) do
  local verses = Quran[surahName]
  if verses then
    for index, verse in ipairs(verses) do
      local formatted = verse .. " [" .. tostring(index) .. "] - " .. surahName
      table.insert(allVerses, formatted)
      verseToSurahMap[formatted] = {surah = surahName, number = index}
    end
  end
end

surahs = {}
for _, name in ipairs(ordered_surahs) do
  if Quran[name] then
    table.insert(surahs, name)
  end
end

adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, surahs)
list.setAdapter(adapter)
filteredItems = surahs
currentItems = surahs
is_bahth_tam = false

search(search_list_id, currentItems)
search(search_quran_id, allVerses)

list.onItemClick = function(parent, view, position)
  if currentItems == surahs then
    local selected = view.text
    local verses = Quran[selected] or {}

    local formattedVerses = {}
    for i, verse in ipairs(verses) do
      local formatted = verse .. " [" .. tostring(i) .. "] - " .. selected
      table.insert(formattedVerses, formatted)
      verseToSurahMap[formatted] = {surah = selected, number = i}
    end

    adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, formattedVerses)
    list.setAdapter(adapter)
    filteredItems = formattedVerses
    currentItems = formattedVerses
    is_bahth_tam = false
    search(search_list_id, currentItems)
  else
    dlg.dismiss()
    task(400, function()
      service.insertText(service.getEditText(), view.text .. "\n")
    end)
  end
end

list.onItemLongClick = function(parent, view, position)
  service.copy(view.text)
  service.speak("تم النسخ")
  return true
end

customTitle.onClick = function()
  if is_bahth_tam and #filteredItems > 0 then
    local allResults = table.concat(filteredItems, "\n")
    service.copy(allResults)
    service.speak("تم النسخ")
  elseif is_bahth_tam and #filteredItems == 0 then
    service.speak("لا توجد نتائج للنسخ")
  else
    service.speak("لم يتم إجراء بحث بعد")
  end
end

customTitle.onLongClick = function()
  if is_bahth_tam and #filteredItems > 0 then
    local extra = {}
    for _, item in ipairs(filteredItems) do
      local info = verseToSurahMap[item]
      if info then
        table.insert(extra, item .. "\n" .. "السورة: " .. info.surah .. "، الآية: " .. info.number)
      else
        table.insert(extra, item)
      end
    end
    service.copy(table.concat(extra, "\n\n"))
    service.speak("تم النسخ الإضافي")
  else
    service.speak("لا توجد نتائج")
  end
  return true
end

dlg.show()